****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidatablest3.sas  
*
* Created (mm/dd/yyyy): 12/19/2014
* Last modified: 06/30/2019
* Version: 3.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create the output tables for type 3 cohort.                                        
*   
*  Program inputs:   
*	-infolder.&TYPE3FILE.
*	-DPLocal.&RUNID._mstr 
*                                     
*  Program outputs:  
*	-DPLOCAL.&RUNID._NumCounts
*	-MSOC.&RUNID._t3_CIDA
*                                                                                                  
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.1      02/23/14	    DM	    Changed the way EPISODES_ANALYSIS and EPISODES_EXPOSURE are
*                                   calculated
*
*    1.2      03/18/15      DM      Changed the way ExpPeriodEndDt is calculated	  
*									
*	 1.3      06/30/16      DM      Added tables with Time to censor data
*     
*	 1.4      07/13/16      DM      Fix for JIRA #QCI-178
*
*    1.5      11/07/16      DM      Fix for JIRA #QRP-279
*
*    1.6	  11/30/16		AP		Fix for JIRA #QRP-282
*
*	 1.7	  05/05/17	    AP		1. Added race and hispanic stratifications
*									2. Moved macro call to group loop
*
* 	 1.8	  10/06/17	    AP		-Added geographic stratifications
*									-Recoded sex A/U value (QRP-453)
*
* 	 2.0	  12/20/17	    AP		1. Combined _t3_ and _t3_censor table (QRP-293)
*									2. Removed comorbidity enrollment requirement (QRP-435)
*
* 	 2.1	  02/01/18	    AP		Removed post-index enrollment requirements (QRP-496)
*
*    2.2      03/14/18      RR		Changed Group length to $40 (QRP-495)
*
* 	 3.0	  05/15/18	    AP		Changed EnrDaysAft to EnrDaysAftInd
*									Moved Metadata file creation to ms_cidanum %forloop
*									Implemented USERSTRATA stratification reorganization
*
* 	 3.1	  06/30/19	    AP		Removed censor stratified output
*								    Supress columns when running tree extraction 
*
***************************************************************************************************;

%macro ms_cidatablest3();

%put =====> MACRO CALLED: ms_cidatablest3 v3.1;

	*Select all output levels relevant to t1_cida/t2_cida;
	data _t&type._levels;
		set userstrata;
		where lowcase(tableID) = "t&type.cida";

		*defensive: convert tte and ttc to tte_value and ttc_value if specified;
		if index(levelvars, 'tte') > 0 & index(levelvars, 'tte_value') = 0 then do;
			levelvars = tranwrd(levelvars, "tte", "tte_value");
		end;
		if index(levelvars, 'ttc') > 0 & index(levelvars, 'ttc_value') = 0 then do;
			levelvars = tranwrd(levelvars, "ttc", "ttc_value");
		end;

		*Remove tte_value as a stratification if running tree extraction;
		%if "&analysis" = "tree" %then %do;
			if index(levelvars, 'tte_value') > 0 then do;
				levelvars = tranwrd(levelvars, "tte_value", "");
			end;
		%end;
	run;

	%isdata(dataset=_t&type._levels);
	%if %eval(&nobs.>0) %then %do;

		** Loop through groups. Will loop through all groups because several stratifiers can differ between groups ;

		%do group=1 %to &Ngroup.;
		
			data _null_;
				set infolder.&COHORTFILE.;
				if _n_ = &Group. then do;
					*Age stratification;
					if missing(agestrat) then do;
						call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
					end;
					else do;
						call symputx("agestrat",agestrat);
					end;

					call symputx("itgroup", cohortgrp);
				end;
			run;

	  	/*******************************************************/
	    /* Macro to compute numerators for each stratification */
	  	/*******************************************************/
	    %macro createNumTableEntries(class=, level=);

	        *One record per patient for the class stratification;
	        proc means data=_mstr nway noprint missing;
		        var NPTS_EXPOSURE
		            NPTS_CENSOR_ELIG 
		            NPTS_CENSOR_DTH
		            NPTS_ANALYSIS
		            EPISODES_EXPOSURE
		            EPISODES_ANALYSIS
		            EVENTS_ANALYSIS_RISK
		            EVENTS_ANALYSIS_CTRL
		            NPTS_CENSOR_NOEVENTS
		            POSTENR_EXPOSURE
					DAYS_FROM_EXPOS_TO_EVENT;

		        class PatId &class.;
		        id patient;
		        output out=_num0(drop=_:) max(NPTS_EXPOSURE)=NPTS_EXPOSURE
		                                  max(NPTS_CENSOR_ELIG)=NPTS_CENSOR_ELIG 
		                                  max(NPTS_CENSOR_DTH)=NPTS_CENSOR_DTH
		                                  max(NPTS_ANALYSIS)=NPTS_ANALYSIS
		                                  sum(EPISODES_EXPOSURE)=EPISODES_EXPOSURE
		                                  sum(EPISODES_ANALYSIS)=EPISODES_ANALYSIS
		                                  sum(EVENTS_ANALYSIS_RISK)=EVENTS_ANALYSIS_RISK
		                                  sum(EVENTS_ANALYSIS_CTRL)=EVENTS_ANALYSIS_CTRL
		                                  max(NPTS_CENSOR_NOEVENTS)=NPTS_CENSOR_NOEVENTS
		                                  min(POSTENR_EXPOSURE)=MINDAYS_POSTENR_EXPOSURE
		                                  max(POSTENR_EXPOSURE)=MAXDAYS_POSTENR_EXPOSURE
		                                  min(DAYS_FROM_EXPOS_TO_EVENT)=MINDAYS_EVENT_ANALYSIS
		                                  max(DAYS_FROM_EXPOS_TO_EVENT)=MAXDAYS_EVENT_ANALYSIS;
	        run;

	        *One record per patient for the class stratification;
	        proc means data=_num0 missing nway noprint;
		        var NPTS_EXPOSURE
		            NPTS_CENSOR_ELIG 
		            NPTS_CENSOR_DTH
		            NPTS_ANALYSIS
		            EPISODES_EXPOSURE
		            EPISODES_ANALYSIS
		            EVENTS_ANALYSIS_RISK
		            EVENTS_ANALYSIS_CTRL
		            NPTS_CENSOR_NOEVENTS
		            MINDAYS_POSTENR_EXPOSURE
		            MAXDAYS_POSTENR_EXPOSURE
		            MINDAYS_EVENT_ANALYSIS
		            MAXDAYS_EVENT_ANALYSIS;
		        class &class.;
		        output out=_outlvl(drop=_:) sum(NPTS_EXPOSURE
		                                        NPTS_CENSOR_ELIG 
		                                        NPTS_CENSOR_DTH
		                                        NPTS_ANALYSIS
		                                        EPISODES_EXPOSURE
		                                        EPISODES_ANALYSIS
		                                        EVENTS_ANALYSIS_RISK
		                                        EVENTS_ANALYSIS_CTRL
		                                        NPTS_CENSOR_NOEVENTS)=
		                                        Min(MINDAYS_POSTENR_EXPOSURE MINDAYS_EVENT_ANALYSIS)=MINDAYS_POSTENR_EXPOSURE MINDAYS_EVENT_ANALYSIS
		                                        Max(MAXDAYS_POSTENR_EXPOSURE MAXDAYS_EVENT_ANALYSIS )=MAXDAYS_POSTENR_EXPOSURE MAXDAYS_EVENT_ANALYSIS;             
	        run;

			*One record per episode for the class stratification;
	        proc means data=_mstr nway noprint missing;
		        var EPISODES_CENSOR_ELIG
					EPISODES_CENSOR_DTH
					EPISODES_CENSOR_NOEVENTS;

		        class &class.;
		        output out=_outlvl2(drop=_:) sum(EPISODES_CENSOR_ELIG
												   EPISODES_CENSOR_DTH
												   EPISODES_CENSOR_NOEVENTS)=;     
	        run;


			%if %str("&class") ne %str("") %then %do;
			data _outlvl;
				merge _outlvl _outlvl2;
				by &class.;
			run;
			%end;
			%else %do; /*overall - no stratifications*/
				/* Set mergenoby to nowarn to avoid triggering an error or w.a.r.n.i.n.g. at this step */
				options mergenoby=nowarn;

				data _outlvl;
					merge _outlvl _outlvl2;
					*no by;
				run;

				/* Reset mergenoby to original value (as stored in macro variable) */
 				options mergenoby=&mergenoby.;
			%end;

		    %if %eval(&s.) = 1 %then %do;
		    	data _numcounts;
		            set _outlvl;
		            format level $3. group $40.;
		            Level="&level.";
					group = "&ITGROUP.";
		     	run;
		     %end;
		     %else %do;
		     	data _numcounts;
		            set _numcounts
		                _outlvl(in=a);
		            if a then do;	
						Level="&level.";
						group = "&ITGROUP.";
					end;
		        run;
		     %end;

			proc datasets nowarn noprint lib=work;
				delete _outlvl: _num0;
			quit;
	    %mend;


		/*********************************/
		/* Compute numerators and output */
		/*********************************/

		%isdata(dataset=_t&type._levels);    
		%do s = 1 %to %eval(&nobs.);
			data _null_;
				set _t&type._levels;
				if _n_ = &s. then do;
					call symputx('levelid', levelid);
					call symputx('levelvars', levelvars);
				end;
			run;

			*Square stratifications vars;
			%ms_squaredtableshell(squarevars =&levelvars.,
							 	 analysisvars=NPTS_EXPOSURE NPTS_CENSOR_ELIG NPTS_CENSOR_DTH NPTS_CENSOR_NOEVENTS NPTS_ANALYSIS EPISODES_EXPOSURE
										   EPISODES_ANALYSIS EVENTS_ANALYSIS_RISK EVENTS_ANALYSIS_CTRL MINDAYS_EVENT_ANALYSIS MAXDAYS_EVENT_ANALYSIS
										   MINDAYS_POSTENR_EXPOSURE MAXDAYS_POSTENR_EXPOSURE EPISODES_CENSOR_ELIG EPISODES_CENSOR_DTH EPISODES_CENSOR_NOEVENTS,
							  	groups="&itgroup.",
								analysis=N);

			data _mstr;
		   		set dplocal.&runid._mstr(in=a where=(group in ("&itgroup.")))
		        _mastersquare(in=b);
		    	if a then do;
			        month=month(indexdt);
			        NPTS_EXPOSURE=1;
			        NPTS_CENSOR_ELIG=CensorEnrol; 
			        NPTS_CENSOR_DTH=CensorDeath;
			        NPTS_ANALYSIS=AnalysisCohort;
					EPISODES_CENSOR_ELIG=CensorEnrol;
					EPISODES_CENSOR_DTH=CensorDeath;		
					EPISODES_EXPOSURE=Pt_Exposure_Episode_Num>0;
					EPISODES_ANALYSIS=(Pt_Analysis_Episode_Num>0 and WinOfevent ne "");
			        EVENTS_ANALYSIS_RISK=(WinOfEvent="RISK");
			        EVENTS_ANALYSIS_CTRL=(WinOfEvent="CTRL");

			        NPTS_CENSOR_NOEVENTS=max(0,AnalysisCohort,CensorDeath,CensorEnrol,EVENTS_ANALYSIS_RISK,EVENTS_ANALYSIS_CTRL)=0;
					EPISODES_CENSOR_NOEVENTS=NPTS_CENSOR_NOEVENTS;

			        TTE_VALUE=DAYS_FROM_EXPOS_TO_EVENT;
					TTC_VALUE=TTC;

					patient=1;
		   		end;
		    run;
					
			/*Summarize data */
			%createNumTableEntries(class=&levelvars.,level=&levelid.);

			proc datasets nowarn noprint lib=work;
				delete _mstr _mastersquare;
			quit;
		%end;

		/*Output to MSOC and DPLOCAL*/
		data _numcounts;
			set _numcounts;
				length race hispanic zip_uncertain $1. zip3 state hhs_reg cb_reg $7.;
				if _n_ = 1 then do;
					dsid = open("_NumCounts");
						if varnum(dsid,"race") = 0 then race ='';
						if varnum(dsid,"hispanic") = 0 then hispanic ='';
						if varnum(dsid,"zip_uncertain") = 0 then zip_uncertain ='';
						if varnum(dsid,"zip3") = 0 then zip3 ='';
						if varnum(dsid,"state") = 0 then state ='';
						if varnum(dsid,"hhs_reg") = 0 then hhs_reg ='';
						if varnum(dsid,"cb_reg") = 0 then cb_reg ='';
						if varnum(dsid,"sex") = 0 then sex ='';
						if varnum(dsid,"agegroup") = 0 then agegroup ='';
						if varnum(dsid,"agegroupnum") = 0 then agegroupnum =.;
						if varnum(dsid,"year") = 0 then year =.;
						if varnum(dsid,"month") = 0 then month =.;
						if varnum(dsid,"tte_value") = 0 then tte_value =.;
						if varnum(dsid,"ttc_value") = 0 then ttc_value =.;
					rc= close(dsid);
				end;
				drop rc dsid;

				/*Supress variables when conducting tree extraction*/
				%if "&analysis" = "tree" %then %do;
					TTE_VALUE = .;
					NPTS_CENSOR_NOEVENTS=.;
					EPISODES_CENSOR_NOEVENTS=.;
					NPTS_ANALYSIS=.;
					EPISODES_ANALYSIS=.;
					EVENTS_ANALYSIS_RISK=.;
					EVENTS_ANALYSIS_CTRL=.;
					MINDAYS_EVENT_ANALYSIS=.;
					MAXDAYS_EVENT_ANALYSIS=.;
				%end;
		run;

		proc sort data=_numcounts;
			by group level sex agegroupnum year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic;
		run; 

    	%if %eval(&group.=1) %then %do;
			data DPLOCAL.&RUNID._numcounts;
	       		set _numcounts;		
	     	run;

			data MSOC.&RUNID._t&type._cida;
				retain group level sex agegroup agegroupnum year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic &covarstrat. TTE_VALUE TTC_VALUE
  				  	   NPTS_EXPOSURE NPTS_ANALYSIS EPISODES_EXPOSURE EPISODES_ANALYSIS NPTS_CENSOR_ELIG NPTS_CENSOR_DTH EVENTS_ANALYSIS_RISK EVENTS_ANALYSIS_CTRL
				   	   MINDAYS_EVENT_ANALYSIS MAXDAYS_EVENT_ANALYSIS MINDAYS_POSTENR_EXPOSURE NPTS_CENSOR_NOEVENTS MINDAYS_POSTENR_EXPOSURE MAXDAYS_POSTENR_EXPOSURE
          		   	   EPISODES_CENSOR_ELIG EPISODES_CENSOR_DTH EPISODES_CENSOR_NOEVENTS;
				keep group level sex agegroup agegroupnum year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic &covarstrat. TTE_VALUE TTC_VALUE
  				  	   NPTS_EXPOSURE NPTS_ANALYSIS EPISODES_EXPOSURE EPISODES_ANALYSIS NPTS_CENSOR_ELIG NPTS_CENSOR_DTH EVENTS_ANALYSIS_RISK EVENTS_ANALYSIS_CTRL
				   	   MINDAYS_EVENT_ANALYSIS MAXDAYS_EVENT_ANALYSIS MINDAYS_POSTENR_EXPOSURE NPTS_CENSOR_NOEVENTS MINDAYS_POSTENR_EXPOSURE MAXDAYS_POSTENR_EXPOSURE
          		   	   EPISODES_CENSOR_ELIG EPISODES_CENSOR_DTH EPISODES_CENSOR_NOEVENTS;
				set _NumCounts;
			run;
	    %end;
	    %else %do;
	        proc append base=DPLOCAL.&RUNID._numcounts
	                    data=_numcounts force;
	        run;
			proc append base=MSOC.&RUNID._t&type._cida
	           			data=_numcounts(keep=group level sex agegroup agegroupnum year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic &covarstrat. TTE_VALUE TTC_VALUE
		  				  	   NPTS_EXPOSURE NPTS_ANALYSIS EPISODES_EXPOSURE EPISODES_ANALYSIS NPTS_CENSOR_ELIG NPTS_CENSOR_DTH EVENTS_ANALYSIS_RISK EVENTS_ANALYSIS_CTRL
						   	   MINDAYS_EVENT_ANALYSIS MAXDAYS_EVENT_ANALYSIS MINDAYS_POSTENR_EXPOSURE NPTS_CENSOR_NOEVENTS MINDAYS_POSTENR_EXPOSURE MAXDAYS_POSTENR_EXPOSURE
		          		   	   EPISODES_CENSOR_ELIG EPISODES_CENSOR_DTH EPISODES_CENSOR_NOEVENTS) force;
			run;
		%end;
	%end; /*loop through groups*/
	%end;

	%put NOTE: ******** END OF MACRO: ms_cidatablest3 v3.1 ********;

%mend ms_cidatablest3;
