****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidatablest5.sas  
*
* Created (mm/dd/yyyy): 06/28/2017
* Last modified: 12/15/2018
* Version: 2.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create the output tables for type 5 cohort                                       
*   
*  Program inputs:    
*	-infolder.&cohortfile.
*	-_PtsMasterList
*	-_Dispensings 
*                                     
*  Program outputs:  
*	-MSOC.&RUNID._t5_CIDA_disp_by_daysupp
*	-MSOC.&RUNID._t5_CIDA_firsteps
*	-MSOC.&RUNID._t5_CIDA_alleps
*	-MSOC.&RUNID._t5_CIDA_episode_duration
*	-MSOC.&RUNID._t5_CIDA_episdur_censor
*	-MSOC.&RUNID._t5_CIDA_gaps
*                                                                                                  
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1		  10/10/17	 	AP		Added geographic stratifications
*
*   1.2       03/14/18      RR		Changed Group length to $40 (QRP-495)
*
*   1.3       03/27/18      AP		Added EpisodeLength per Episode to t5_cida_episdur (QRP-516)
*
*   1.4       04/16/18      AP		Changed Dispensings to Adjusted/Raw Code Count (QRP-305)
*
*	2.0		  05/15/18	 	AP		Reorganized for user selected strata
*
*	2.1		  012/15/18	 	AP		Fixed bug DEV-3026 (changed where to if)
*
***************************************************************************************************;

%macro ms_cidatablest5();

%put =====> MACRO CALLED: ms_cidatablest5 v2.1;

	*Create dataset for each table;
	data _t5levels_disp
		 _t5levels_first
		 _t5levels_alleps
		 _t5levels_episdur
		 _t5levels_censor
		 _t5levels_gaps;
		set userstrata;
		
		if lowcase(tableID) = "t5disp" then output _t5levels_disp;
		if lowcase(tableID) = "t5first" then output _t5levels_first;
		if lowcase(tableID) = "t5alleps" then output _t5levels_alleps;
		if lowcase(tableID) = "t5episdur" then output _t5levels_episdur;
		if lowcase(tableID) = "t5censor" then output _t5levels_censor;
		if lowcase(tableID) = "t5gaps" then output _t5levels_gaps;
	run;

	%let t5dispstrat1 = ;
	%let t5firststrat1 = ;
	%let t5allepsstrat1 = ;
	%let t5episdurstrat1 = ;
	%let t5censorstrat1 = ;
	%let t5gapsstrat1 = ;

	proc sql noprint;
		select distinct lowcase(levelvars) into: t5dispstrat1 separated by ' '
		from _t5levels_disp;
		select distinct lowcase(levelvars) into: t5firststrat1 separated by ' '
		from _t5levels_first;
		select distinct lowcase(levelvars) into: t5allepsstrat1 separated by ' '
		from _t5levels_alleps;
		select distinct lowcase(levelvars) into: t5episdurstrat1 separated by ' '
		from _t5levels_episdur;
		select distinct lowcase(levelvars) into: t5censorstrat1 separated by ' '
		from _t5levels_censor;
		select distinct lowcase(levelvars) into: t5gapsstrat1 separated by ' '
		from _t5levels_gaps;
	quit;

	%nonrep(invar = t5dispstrat1, outvar = t5dispstrat);
	%nonrep(invar = t5firststrat1, outvar = t5firststrat);
	%nonrep(invar = t5allepsstrat1, outvar = t5allepsstrat);
	%nonrep(invar = t5episdurstrat1, outvar = t5episdurstrat);
	%nonrep(invar = t5censorstrat1, outvar = t5censorstrat);
	%nonrep(invar = t5gapsstrat1, outvar = t5gapsstrat);


    /*********************************************************/
	/* Note when squaring: Continuous values are not squared */
    /*********************************************************/

	** Loop through groups. To prevent unnecessary looping, only loop if AgeGroup differs between groups ** ;

	%do Group=1 %to &cidatablesgroup.;

		data _null_;
			set infolder.&COHORTFILE.;
			if _n_ = &Group. then do;
				call symputx('ITGROUP',strip(cohortgrp));            

			*Age stratification;
			if missing(agestrat) then do;
				call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
			end;
			else do;
				call symputx("agestrat",agestrat);
			end;

			call symputx("itgroup", cohortgrp);
			end;
		run;

		*if running all groups, grouplist is blank, otherwise only aggregate data for current group;
		%let grouplist = ;
 		%if %eval(&count_agegroup. >1) %then %do;
			%let grouplist = "&itgroup."; 
		%end;

	
		/****************************/
		/* t5_cida_disp_by_daysupp	*/
		/****************************/
	    %macro create_t5_cida_disp_by_daysupp(class=,level=);
	        proc means data=_disp nway noprint missing;
		        var dispensing;
		        class group &class.;
		        id dispensing;
		        output out=disp_daysup_strat(drop=_:) sum(dispensing)=AdjustedCodeCount;                                   
	        run;

			%if %eval(&s.=1) %then %do;
		    	data _t5_cida_disp_by_daysupp;
		            set disp_daysup_strat(drop=dispensing);
		            format level $3.;
		            Level="&level.";
		    	run;
			%end;
		    %else %do;
		    	data _t5_cida_disp_by_daysupp;
		        	set _t5_cida_disp_by_daysupp
		            	disp_daysup_strat(drop=dispensing in=a);
		            if a then do;	
						level="&level.";
					end;
				run;
			%end;

			proc datasets lib=work noprint nowarn;
				delete disp_daysup_strat; 
			quit;
	    %mend;

		%isdata(dataset=_t5levels_disp);    
		%if %eval(&nobs.>0) %then %do;
			%do s = 1 %to %eval(&nobs.);
				data _null_;
					set _t5levels_disp;
					if _n_ = &s. then do;
						call symputx('levelid', levelid);
						call symputx('levelvars', levelvars);
					end;
				run;

				*Square table;
				%ms_squaredtableshell(squarevars =&levelvars.,
								      analysisvars=disp_rxsup,
								      groups=&grouplist.,
									  analysis=N);
					
				/***********************/
				/*Dispensing level data*/
				/***********************/
				data _disp(rename=disp_rxsup=daysupp);
					set dplocal.&runid._dispensings(in=a %if %str(&grouplist.) ne () %then %do; where=(group in (&grouplist.))%end;)
				        _mastersquare(in=b);
				    if a then do;
				        dispensing=1;
				    end;
				    else do;
				        dispensing=0;    
				    end;
				run;

				/*Summarize data */
				%create_t5_cida_disp_by_daysupp(class=&levelvars.,level=&levelid.);

				proc datasets nowarn noprint lib=work;
					delete _disp _mastersquare;
				quit;
			%end;

			*sort;
			proc sort data=_t5_cida_disp_by_daysupp;
				by group level &t5dispstrat.;
			run;

			/*** Output to MSOC ***/
			%if %eval(&group.=1) %then %do;
				data msoc.&runid._t5_cida_disp_by_daysupp;
					retain group level &t5dispstrat. AdjustedCodeCount;
					set _t5_cida_disp_by_daysupp;
				run;
			%end;
			%else %do;
		 		proc append base=msoc.&runid._t5_cida_disp_by_daysupp
			                    data=_t5_cida_disp_by_daysupp force;
			    run;
			%end;
		%end;


		/****************************/
		/* t5_cida_from_studystart	*/
		/****************************/
		%macro create_t5_cida_from_studystart(data=, class=,level=);
			proc means data=_eps nway noprint missing;
	        	var Patient totrxsup AdjustedDisp RawDisp ;
	        	class group PatId &class.;
	        	id patient;
	        	output out=_num0(drop=_:) max(Patient)=Npts
	                                  sum(Patient)=Episodes           
	                                  sum(totrxsup)=Daysupp         
 									  sum(AdjustedDisp)=AdjustedCodeCount
                                  	  sum(RawDisp)=RawCodeCount;            
	        run;

			proc means data=_num0 nway noprint missing;
				var Npts Episodes Daysupp AdjustedCodeCount RawCodeCount;
			    class group &class.;
			    id patient;
			    output out=_t5studystart_strat(drop=_:) sum=;                                  
			run;

			%if %eval(&s.=1) %then %do;
			    data _t5_cida&data.;
			    	set _t5studystart_strat(drop=patient);
			        format level $3.;
			        Level="&level.";
			    run;
			%end;
			%else %do;
				data _t5_cida&data.;
			    	set _t5_cida&data.
			        	_t5studystart_strat(drop=patient in=a);
			            if a then do;	
							Level="&level.";
						end;
				run;
			%end;

			proc datasets lib=work noprint nowarn; delete _t5studystart_strat _num0; quit;
		%mend;

		%isdata(dataset=_t5levels_first);    
		%if %eval(&nobs.>0) %then %do;

			****FirstEps****;
			data _firsteps;
				set dplocal.&runid._mstr(in=a %if %str(&grouplist.) ne () %then %do; where=(group in (&grouplist.))%end;);
				if episodenum = 1;
			run;

			%do s = 1 %to %eval(&nobs.);
				data _null_;
					set _t5levels_first;
					if _n_ = &s. then do;
						call symputx('levelid', levelid);
						call symputx('levelvars', levelvars);
					end;
				run;

				*Square table;
				%ms_squaredtableshell(squarevars =&levelvars.,
								      analysisvars=totrxsup AdjustedDisp RawDisp,
								      groups=&grouplist.,
									  analysis=N);

				/********************/
				/*Episode-level data*/
				/********************/
				data _eps;
					set _firsteps (in=a)
				        _mastersquare(in=b);
				    if a then do;
				        patient=1;
				    end;
				    else do;
				        patient=0;    
				    end;
				run;
					
				/*Summarize data */
				%create_t5_cida_from_studystart(data=_firsteps, class=&levelvars.,level=&levelid.);

				proc datasets nowarn noprint lib=work;
					delete _eps _mastersquare;
				quit;
			%end;

			*sort;
			proc sort data=_t5_cida_firsteps;
				by group level &t5firststrat.;
			run;

			/*** Output to MSOC ***/
			%if %eval(&group.=1) %then %do;
				data msoc.&runid._t5_cida_firsteps;
					retain group level &t5firststrat. episodes npts daysupp adjustedcodecount rawcodecount;
					set _t5_cida_firsteps;
				run;
			%end;
			%else %do;
		 		proc append base=msoc.&runid._t5_cida_firsteps
			                    data=_t5_cida_firsteps force;
			    run;
			%end;
		%end;


		%isdata(dataset=_t5levels_alleps);    
		%if %eval(&nobs.>0) %then %do;

			****AllEps****;
			%do s = 1 %to %eval(&nobs.);
				data _null_;
					set _t5levels_alleps;
					if _n_ = &s. then do;
						call symputx('levelid', levelid);
						call symputx('levelvars', levelvars);
					end;
				run;

				*Square table;
				%ms_squaredtableshell(squarevars =&levelvars.,
								      analysisvars=totrxsup AdjustedDisp RawDisp,
								      groups=&grouplist.,
									  analysis=N);

				/********************/
				/*Episode-level data*/
				/********************/
				data _eps;
					set dplocal.&runid._mstr(in=a %if %str(&grouplist.) ne () %then %do; where=(group in (&grouplist.))%end;)
				        _mastersquare(in=b);
				    if a then do;
				        patient=1;
				    end;
				    else do;
				        patient=0;    
				    end;
				run;
						
				/*Summarize data */
				%create_t5_cida_from_studystart(data=_alleps, class=&levelvars.,level=&levelid.);

				proc datasets nowarn noprint lib=work;
					delete _eps _mastersquare;
				quit;
			%end;

			*sort;
			proc sort data=_t5_cida_alleps;
				by group level &t5allepsstrat.;
			run;

			/*** Output to MSOC ***/
			%if %eval(&group.=1) %then %do;
				data msoc.&runid._t5_cida_alleps;
					retain group level &t5allepsstrat. episodes npts daysupp adjustedcodecount rawcodecount;
					set _t5_cida_alleps;
				run;
			%end;
			%else %do;
		 		proc append base=msoc.&runid._t5_cida_alleps
			                    data=_t5_cida_alleps force;
			    run;
			%end;
		%end;

		/****************************/
		/* t5_cida_episode_duration	*/
		/****************************/
	  	%macro t5_cida_episode_duration_gaps(class=,level=, dataout=);
	        proc means data=_episdur nway noprint missing;
		        var patient;
		        class group patid &class.;
		        id patient;
		        output out=episdur_strat(drop=_:) sum(patient)=Episodes max(patient)=npts;                                  
	        run;

			proc means data=episdur_strat nway noprint missing;
				var Npts Episodes;
			    class group &class.;
			    id patient;
			    output out=episdur_strat(drop=_:) sum=;                                  
			run;

			%if %eval(&s.=1) %then %do;
		    	data &dataout.;
		            set episdur_strat(drop=patient);
		            format level $3.;
		            Level="&level.";
		    	run;
			%end;
		    %else %do;
		    	data &dataout.;
		        	set &dataout.
		                episdur_strat(drop=patient in=a);
		                if a then do;	
							Level="&level.";
						end;
				run;
			%end;

			proc datasets lib=work noprint nowarn; delete episdur_strat; quit;
		%mend;

		/*******************/
		/* t5_cida_episdur */
		/*******************/
		%isdata(dataset=_t5levels_episdur);    
		%if %eval(&nobs.>0) %then %do;
			%do s = 1 %to %eval(&nobs.);
				data _null_;
					set _t5levels_episdur;
					if _n_ = &s. then do;
						call symputx('levelid', levelid);
						call symputx('levelvars', levelvars);
					end;
				run;
					
				%ms_squaredtableshell(squarevars =&levelvars.,
								      analysisvars=,
								      groups=&grouplist.,
									  analysis=N);

				data _episdur;
					set dplocal.&runid._mstr(in=a %if %str(&grouplist.) ne () %then %do; where=(group in (&grouplist.))%end;)
				        _mastersquare(in=b);
				    if a then do;
				        patient=1;
				    end;
				    else do;
				        patient=0;    
				    end;
				run;
						
				/*Summarize data */
				%t5_cida_episode_duration_gaps(class=&levelvars., level=&levelid., dataout=_t5_cida_episdur);

				proc datasets nowarn noprint lib=work;
					delete _episdur _mastersquare;
				quit;
			%end;

			*sort;
			proc sort data=_t5_cida_episdur;
				by group level &t5episdurstrat.;
			run;

			/*** Output to MSOC ***/
			%if %eval(&group.=1) %then %do;
				data msoc.&runid._t5_cida_episdur;
					retain group level &t5episdurstrat. episodes npts;
					set _t5_cida_episdur;
				run;
			%end;
			%else %do;
		 		proc append base=msoc.&runid._t5_cida_episdur
			                data=_t5_cida_episdur force;
			    run;
			%end;
		%end;	

		/****************/
		/* t5_cida_gaps	*/
		/****************/
		%isdata(dataset=_t5levels_gaps);    
		%if %eval(&nobs.>0) %then %do;
			%do s = 1 %to %eval(&nobs.);
				data _null_;
					set _t5levels_gaps;
					if _n_ = &s. then do;
						call symputx('levelid', levelid);
						call symputx('levelvars', levelvars);
					end;
				run;

				%ms_squaredtableshell(squarevars =&levelvars.,
								      analysisvars=,
								      groups=&grouplist.,
									  analysis=N);

				data _episdur;
					set dplocal.&runid._mstr(in=a %if %str(&grouplist.) ne () %then %do; where=(group in (&grouplist.))%end;)
				        _mastersquare(in=b);
				    if a then do;
				        patient=1;
				    end;
				    else do;
				        patient=0;    
				    end;
				run;
						
				/*Summarize data */
				%t5_cida_episode_duration_gaps(class=&levelvars., level=&levelid., dataout=_t5_cida_gaps);

				proc datasets nowarn noprint lib=work;
					delete _episdur _mastersquare;
				quit;
			%end;

			*sort;
			proc sort data=_t5_cida_gaps;
				by group level &t5gapsstrat.;
			run;

			/*** Output to MSOC ***/
			%if %eval(&group.=1) %then %do;
				data msoc.&runid._t5_cida_gaps;
					retain group level &t5gapsstrat. episodes;
					set _t5_cida_gaps;
				run;
			%end;
			%else %do;
		 		proc append base=msoc.&runid._t5_cida_gaps
			                data=_t5_cida_gaps force;
			    run;
			%end;
		%end;

		/****************************/
		/* t5_cida_episdur_censor	*/
		/****************************/
		%macro t5_cida_episdur_censor(class=,level=);
	        proc means data=_allepscensor nway noprint missing;
		        var patient;
		        class group patid &class.;
		        id patient;
		        output out=_censtrat(drop=_:)	sum(patient)=Episodes
												max(patient)=Npts
											  	sum(episode_cens_elig)=cens_elig
												sum(episode_cens_dth)=cens_dth
												sum(episode_cens_dpend)=cens_dpend
												sum(episode_cens_qryend)=cens_qryend
												sum(episode_cens_episend)=cens_episend
												sum(episode_cens_spec)=cens_spec;
	        run;

			proc means data=_censtrat nway noprint missing;
				var Npts Episodes cens_elig cens_dth cens_dpend cens_qryend cens_episend cens_spec;
			    class group &class.;
			    id patient;
			    output out=_censtrat(drop=_:) sum=;                                  
			run;

			%if %eval(&s.=1) %then %do;
		    	data _t5_cida_episdur_censor;
		            set _censtrat(drop=patient);
		            format level $3.;
		            Level="&level.";
		    	run;
			%end;
		    %else %do;
		    	data _t5_cida_episdur_censor;
		        	set _t5_cida_episdur_censor
		                _censtrat(drop=patient in=a);
		                if a then do;	
							Level="&level.";
						end;
				run;
			%end;

			proc datasets lib=work noprint nowarn; delete _censtrat; quit;
		%mend;

		%isdata(dataset=_t5levels_censor);    
		%if %eval(&nobs.>0) %then %do;
			%do s = 1 %to %eval(&nobs.);
				data _null_;
					set _t5levels_censor;
					if _n_ = &s. then do;
						call symputx('levelid', levelid);
						call symputx('levelvars', levelvars);
					end;
				run;
						
				%ms_squaredtableshell(squarevars =&levelvars.,
								      analysisvars=episode,
								      groups=&grouplist.,
									  analysis=N);

				data _allepscensor;
					set dplocal.&runid._mstr(in=a %if %str(&grouplist.) ne () %then %do; where=(group in (&grouplist.))%end;)
				        _mastersquare(in=b);
				    if a then do;
				        patient=1;
				    end;
				    else do;
				        patient=0;   
						episode_cens_elig=0;
					 	episode_cens_dth=0;
						episode_cens_dpend=0;
						episode_cens_qryend=0;
						episode_cens_episend=0;
						episode_cens_spec=0;
				    end;
				run;

				/*Summarize data */
				%t5_cida_episdur_censor(class=&levelvars., level=&levelid.);
			%end;

			*sort;
			proc sort data=_t5_cida_episdur_censor;
				by group level &t5censorstrat.;
			run;

			/*** Output to MSOC ***/
			%if %eval(&group.=1) %then %do;
				data msoc.&runid._t5_cida_episdur_censor;
					retain group level &t5censorstrat. episodes cens:;
					set _t5_cida_episdur_censor;
				run;
			%end;
			%else %do;
		 		proc append base=msoc.&runid._t5_cida_episdur_censor
			                data=_t5_cida_episdur_censor force;
			    run;
			%end;
		%end;	

	%end; *loop through groups;

%put NOTE: ******** END OF MACRO: ms_cidatablest5  v2.1 ********;

%mend ms_cidatablest5;
