****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_evalsecondaryepi.sas  
*
* Created (mm/dd/yyyy): 05/31/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: Evaluate secondary episodes with relation to multiple events, overlap and adherence                                   
*   
*  Program inputs:  
*             	                        
*  Program outputs:    
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_evalsecondaryepi();

	%put =====> MACRO CALLED: ms_evalsecondaryepi v1.0; 

	*Merge with ITSFILE, if ITS requested;
	%ISDATA(dataset=infolder.&ITSFILE.);	
	%if %eval(&nobs.>0) %then %do;
		%let ITSANALYSIS = 1;

		proc sql noprint undo_policy=none;
			create table secondaryinputfile as
			select sec.*
				 , its.adherenceID
				 , its.interventiondate1
				 , its.interventiondate2
				 , its.interval
				 , its.its_enrollment
			from secondaryinputfile as sec
			left join infolder.&itsfile as its
			on sec.analysisgrp = its.analysisgrp;
		quit;
	%end;
	%else %do;			
		data secondaryinputfile;	
			set secondaryinputfile;
				adherenceID = .;
				interventiondate1 = .;
				interventiondate2 = .;
				interval = '';
				its_enrollment = '';
		run;
	%end;

   	/*1. Loop through Analysis Groups*/	
	%isdata(dataset=secondaryinputfile); 
	data _null_;
    call symputx("num_order","&nobs.");
    run;
	%put The number of analysis groups is &num_order.;

	%global cutoff_varlist num_cutoffcat;

	%do r = 1 %to &num_order.;

		%ms_starttimer(timestart=secondaryruntime);

		/*reset macro vars*/
		%let analysisgrp = ;
		%let primary = ;
		%let secondary = ;
		%let outcome = ;
		%let obsfrom = ;
		%let obsto = ;
		%let obsfromanchor = ;
		%let obstoanchor = ;
		%let episodenum = ;

		%let minadherence = ;
		%let minadherence_scale = ;
		%let cutoffcat = ;
		%let cutoffcat_scale = ;
		%let INTERVENTIONDATE1 = .;
		%let INTERVENTIONDATE2 = .;
		%let tte_output_cat = ;

		%let calculate_adherence = N;
		%let num_adherence = 0;

		/*1 Assign macro variables*/
		data _null_;
			set secondaryinputfile;
			if _n_ = &r. then do;
				call symputx("analysisgrp", analysisgrp);
				call symputx("primary", lowcase(primary));
				call symputx("secondary", lowcase(secondary));
				call symputx("obsfrom", obsfrom);
				call symputx("obsto", obsto);
				call symputx("obsfromanchor", lowcase(obsfromanchor));
				call symputx("obstoanchor", lowcase(obstoanchor));
				call symputx("minadherence", minadherence);
				call symputx("minadherence_scale", lowcase(minadherence_scale));
				call symputx("INTERVENTIONDATE1", INTERVENTIONDATE1);
				call symputx("INTERVENTIONDATE2", INTERVENTIONDATE2);
				call symputx("tte_output_cat", tte_output_cat);

				%if %eval(&multievent.=1)%then %do;
				call symputx("episodenum", episodenum);
				%end;

				%if %eval(&overlap.=1) %then %do;
				call symputx("cutoffcat", cutoffcat);
				call symputx("cutoffcat_scale", lowcase(cutoffcat_scale));
				%end;
			end;
		run;
		%put now looping on order: &r.;
		%put group name: &analysisgrp.;
		%put primary episode: &primary.;
		%put secondary episode: &secondary.;
		%put Start of observation window: &obsfrom. days from &obsfromanchor.;
		%put End of observation window: &obsto. days from &obstoanchor.;
		%put Minimum adherence episodes per patient: &minadherence. &minadherence_scale.;

		%if %eval(&multievent.=1)%then %do;
		%put Calculating time to &episodenum. episode;
		%end;

		%if %eval(&overlap.=1) %then %do;
		%put cut-off categories: &cutoffcat &cutoffcat_scale.;
		%end;

		%put intervention date 1: &INTERVENTIONDATE1.;
		%put intervention date 2: &INTERVENTIONDATE2.;

		/*Number of intervention dates*/
		%let maxdates = 0;
		%if "&interventiondate2" ne "." %then %do; /*2 intervention dates*/
			%let maxdates = 2;
		%end;
		%else %if "&interventiondate1" ne "." %then %do; /*1 intervention date*/
			%let maxdates = 1;
		%end;

		/*2. If more than 1 secondary GROUP, create secondary episodes by collapsing the individual group episodes into non-overlapping episodes.*/
		%if %eval(%sysfunc(countw(&secondary.)) > 1) %then %do;
			data epis_to_collapse;
				set dplocal.&runid._mstr;
				where lowcase(group) in 
					(
					%do grpp = 1 %to %sysfunc(countw(&secondary.));
					%let currgrp = %scan(&secondary., &grpp.);
					%if &grpp. ne 1 %then %do; , %end; "&currgrp."
					%end;	
					);
			run;

			*Manage quantities occuring on the same day;
			proc means noprint data = epis_to_collapse nway missing;
				var tte adjustedDisp RawDisp totrxsup totrxamt;
				class PatId indexdt ; 	
				output out=epis_to_collapse(drop=_:) max(tte)=
			                                	     sum(adjustedDisp)=
					                         	     sum(RawDisp)=
											 		 sum(totrxsup)= 
													 sum(totrxamt)=;
			run;	

			data epis_to_collapse;
				 set epis_to_collapse;
				 ExpireDt = indexdt + tte - 1;
			run;

			proc sort data=epis_to_collapse; 
				by patid indexdt expiredt; 
			run;

			*Collapse episodes;
				data epis_to_collapse;
					set epis_to_collapse;
					by patid;
					if first.PatId then do;
						LRunOutDate=Expiredt;
						gap=.;
						episode=1;
					end;
					else do;
						gap = indexdt - LRunOutDate - 1;
						if gap > 0 then do;
						episode=episode+1;
						LRunOutDate = Expiredt;
					end;
						else LRunOutDate = max(Expiredt,LRunOutDate);
					end;

					group = "&secondary.";
					format LRunOutDate date9.;
					retain episode LRunOutDate;
				run;

				proc means noprint data=epis_to_collapse nway missing;
					var indexdt ExpireDt adjustedDisp RawDisp totrxsup totrxamt;
					class group patid episode;
					output out=secondary_episodes(drop=_:) min(indexdt)=
														   max(ExpireDt)=EpisodeEndDt
			                                	    	   sum(adjustedDisp)=
					                         	     	   sum(RawDisp)=
														   sum(totrxamt)= 
														   sum(totrxsup)=;
				run;

				data secondary_episodes;
					set secondary_episodes;
					tte = episodeenddt - indexdt + 1;
				run;
		%end;
		%else %do;
			data secondary_episodes (keep=group patid episodeenddt indexdt adjustedDisp rawdisp totrxsup totrxamt tte);
				set dplocal.&runid._mstr;
				where lowcase(group) = "&secondary.";
			run;
		%end;

		/*3. Remove episodes primary episodes that do not meet obsfrom - obsto enrollment criteria*/
			%let obs_from_index = .;
			%let obs_from_epiend = .;
			%let obs_to_index = .;
			%let obs_to_epiend = .;

			%if "&obsfromanchor." = "index"  & %eval(. < &obsfrom. < 0) %then %do;
				%let obs_from_index = &obsfrom.;
			%end;
			%if "&obsfromanchor." = "episodeend"  & %eval(. < &obsfrom. < 0) %then %do;
				%let obs_from_epiend = &obsfrom.;
			%end;
			%if "&obstoanchor." = "index"  & %eval(&obsto. > 0) %then %do;
				%let obs_to_index = &obsto.;
			%end;
			%if "&obstoanchor." = "episodeend" %then %do;
				%let obs_to_epiend = &obsto.;
			%end;
			%put &obs_from_index &obs_from_epiend &obs_to_index &obs_to_epiend;

			*TTE cat;
			%let tte_output_cat1 = %sysfunc(tranwrd(&tte_output_cat., +, -999999999));
			%let numttecat= %sysfunc(countw(&tte_output_cat., ' '));
			%put &tte_output_cat. &numttecat.;


			%macro createepisodes(truncatedate=);

				data primary_episodes;
					length tte_cat $20;
					set dplocal.&runid._mstr;
					where upcase(group) = upcase("&primary.");

					*calculate tte_cat;
					do i=1 to &numttecat.;        
						if scan(scan("&tte_output_cat1.",i,' '),1, '-') <= tte <= scan(scan("&tte_output_cat1.",i,' '),2, '-') then do;
							tte_cat = scan("&tte_output_cat.",i,' ');
							tte_catnum = i;
							leave;
					    end;
					end;

					if missing(tte_cat) then do;
						tte_cat = 'Other';
						tte_catnum = &numttecat.+1;
					end;

					/*Truncate if truncation specified - ITS analysis*/
					indenominator = 1;
					%if &truncatedate. = Y %then %do;
						if &interventiondate1. ne . then do;
							if indexdt <= &interventiondate1. and  EpisodeEndDt >= &interventiondate1. then EpisodeEnddt = &interventiondate1.;
						end;
						if &interventiondate2. ne . then do;
							if indexdt <= &interventiondate2. and EpisodeEnddt >= &interventiondate2. then EpisodeEnddt = &interventiondate2.;
						end;

						/*Re-evaluate minepisdur for truncated episode*/
						if EpisodeEndDt-IndexDt + 1 < minepisdur then indenominator = 0;
					%end;

					if Enr_Start <= IndexDt-max(0,-1*&obs_from_index.) then MinEnrolMet=1;
					if Enr_Start <= EpisodeEndDt-max(0,-1*&obs_from_epiend.) then MinEnrolMet1=1;
					if Enr_End >= IndexDt+max(0,&obs_to_index.) then MinEnrolMet2=1;
					if Enr_End >= EpisodeEndDt + max(0, &obs_to_epiend.) then MinEnrolMet3=1;

					/*Define observation window*/
					if "&obsfromanchor." = "index" then do;
						obs_start = indexdt + &obsfrom.;
					end;
					if "&obsfromanchor." = "episodeend" then do;
						obs_start = episodeenddt + &obsfrom.;
					end;
					if "&obstoanchor." = "index" then do;
						obs_end = indexdt + &obsto.;
					end;				                            
					if "&obstoanchor." = "episodeend" then do;
						obs_end = episodeenddt + &obsto.;
					end;

					/*Truncate risk window*/
					%if &truncatedate. = Y %then %do;
					if indexdt <= &interventiondate1. and obs_end > &interventiondate1. then obs_end = &interventiondate1.;
					if indexdt <= &interventiondate2. and obs_end > &interventiondate2. then obs_end = &interventiondate2.;

					/*Defensive - remove people with no time at risk*/
					if obs_end < obs_start then indenominator = 0;
					%end;

					format obs_start obs_end date9.;

					if sum(MinEnrolMet, MinEnrolMet1, MinEnrolMet2, MinEnrolMet3) < 4 then delete;
				run;

			/*4. Compute the variable primaryep_num - counter for the number of primary episodes per PatID */
				proc sort data=primary_episodes;
					by patid indexdt;
				run;

				data primary_episodes;
					set primary_episodes;
					by patid indexdt;

					if first.patid then primaryep_num = 1;
					else primaryep_num = primaryep_num + 1;

					retain primaryep_num;
				run;

			/*5. For each primary episode, output an observation for each secondary episode that occurs during the risk window */
				proc sql noprint;
					create table _primary_secondary_eps as
					select distinct x.*
						, y.indexdt as secondary_indexdt 
						, y.episodeenddt as secondary_episodeenddt
						, y.tte as secondary_tte
						, y.RawDisp as secondary_rawdisp
						, y.adjusteddisp as secondary_adjusteddisp
						, y.totrxsup as secondary_totrxsup
						, y.totrxamt as secondary_totrxamt
						, case when %MS_PeriodsOverlap(period1=obs_start obs_end, period2=secondary_indexdt secondary_episodeenddt) then 1
						else 0
						end as inriskwindow
					from primary_episodes as x
					left join secondary_episodes as y
					on x.patid = y.patid
					order by patid, primaryep_num, secondary_indexdt;
				quit;

				/*For each primary episode, select: all episodes in the risk window. If no episodes in the risk window, keep primary episode*/	
				proc means data=_primary_secondary_eps noprint nway;
					var inriskwindow;
					class patid primaryep_num;
					output out = _number_epi_in_overlap(drop=_:) sum=epi_count;
				run;

				proc sql noprint undo_policy=none;
					create table _primary_secondary_eps as
					select x.*,
					y.epi_count
					from _primary_secondary_eps as x,
					_number_epi_in_overlap as y
					where x.patid = y.patid and x.primaryep_num = y.primaryep_num;
				quit;

				data _primary_secondary_eps(drop=epi_count);
					set _primary_secondary_eps;
					by patid primaryep_num;

					if first.primaryep_num then do;
						if inriskwindow = 1 then output;
						else do;
							if epi_count = 0 then do
							secondary_indexdt = .;
							secondary_episodeenddt = .; 
							secondary_tte = .;
							secondary_rawdisp = .;
							secondary_adjusteddisp = .;
							secondary_totrxsup = .;
							secondary_totrxamt = .;
							output;
							end;
						end;
					end; 
					else do;
						if inriskwindow = 1 then output;
					end;
				run;

				data _primary_secondary_eps;
					set _primary_secondary_eps;
					by patid primaryep_num secondary_indexdt;

						/*Epi_num = counter for the secondary episode number per primary episode*/
						if first.primaryep_num then epi_num = 1;
						else epi_num = epi_num + 1;
						retain epi_num;			
				run;

				/*Multiple Events Analysis*/
				%if %eval(&multievent.=1) %then %do;
					data _primary_secondary_eps(drop=lEpiend);	
						set _primary_secondary_eps;
							by patid primaryep_num;
						
						/*Recode epi_num to 0 if no secondary episode*/
						if secondary_indexdt = . then epi_num = 0;

						/*time_to_epi*/
						if epi_num = &episodenum. then do;
							%if %eval(&obsfrom >=0) %then %do;
								if "&obsfromanchor." = "index" then do;
									time_to_epi= secondary_indexdt - indexdt;
								end;
								if "&obsfromanchor." = "episodeend" then do;
									time_to_epi= Secondary_Indexdt - EpisodeEndDt;
								end;
							%end;
							%else %do;
								time_to_epi = .;
							%end;

							/*defensive*/
							if time_to_epi <0 then time_to_epi = .;
						end;

						/*epi_gap*/
						if epi_num <= &episodenum. then do;
							epi_gap = .;
							lEpiend = secondary_episodeenddt;
						end;
						else do;
							epi_gap = secondary_indexdt - lEpiend - 1;
							lEpiend = secondary_episodeenddt;
						end;
						retain lEpiend;		

						/*output episode gap between &episodenum to &episodenum +1 AND &episodenum +1 to &episodenum +2*/
						if epi_num = %eval(&episodenum. + 1) then epi_gap1 = epi_gap; else epi_gap1 = .;
						if epi_num = %eval(&episodenum. + 2) then epi_gap2 = epi_gap; else epi_gap2 = .;

						if last.primaryep_num then do;
							if inriskwindow = 1 and episodeenddt > secondary_episodeenddt then time_to_censor = episodeenddt - secondary_episodeenddt - 1;
							else time_to_censor = .;
						end;
						else do;
							time_to_censor = .;
						end;
					run;

					/*compute epi_count and collapse to 1 obs per primaryep_num - this dataset will be used to calculate adherence */
					proc means data=_primary_secondary_eps noprint nway missing;
						var epi_num epi_gap time_to_epi time_to_censor;
						class patid primaryep_num tte;
						output out=_oneperprimary(drop=_:) max(epi_num)=epi_count
															min(epi_gap)=min_epigap
															max(epi_gap)=max_epigap
															max(epi_gap1)=epi_gap1
															max(epi_gap2)=epi_gap2
															max(time_to_epi) = time_to_epi
															max(time_to_censor)= time_to_censor;
					run;
				%end; /*Multiple Events Analysis*/

				/*Overlap Episode Analysis*/
				%if %eval(&overlap.=1) %then %do;
					data _primary_secondary_eps;	
						set _primary_secondary_eps;
							by patid primaryep_num;

						/*Recode epi_num to 0 if no secondary episode*/
						if secondary_indexdt = . then epi_num = 0;

						/*Count number of days of overlap between obs_start/obs_end and secondary_indexdt/secondary_episodeenddt for each secondary episode*/
							if epi_num >0 then do;
								max_overlap = obs_end - obs_start + 1; /*Length of risk window*/
									/*Remove time prior to secondary episode start*/
									if obs_start < secondary_indexdt then subtract1 = secondary_indexdt - obs_start;
										else subtract1 = 0;
									/*Remove time after secondary episode end*/
									if obs_end > secondary_episodeenddt then subtract2 = obs_end - secondary_episodeenddt;
										else subtract2 = 0;

									days_overlap = max_overlap - subtract1 - subtract2;
							end;
							else do;
								days_overlap = 0;
							end;

							drop subtract:;
					run;

					/*compute epi_count and total_days_overlap and collapse to 1 obs per primaryep_num - this dataset will be used to calculate overlap/adherence*/
					proc means data=_primary_secondary_eps noprint nway missing;
						var epi_num days_overlap secondary_tte secondary_adjusteddisp secondary_rawdisp secondary_totrxsup secondary_totrxamt ;
						class patid primaryep_num max_overlap indexdt;
						output out=_oneperprimary(drop=_:) max(epi_num)=epi_count
															sum(days_overlap)=total_days_overlap
															sum(secondary_tte)=tte2
															sum(secondary_adjusteddisp)=AdjustedDisp2
															sum(secondary_rawdisp)=RawDisp2
															sum(secondary_totrxsup)=totrxsup2
															sum(secondary_totrxamt)=totrxamt2;		
					run;

					/*Create array with cutoff values:
						1. convert dash to underscore
						2. convert >/< to chars*/

					%let cutoff_varlist = ;
					%let num_cutoffcat = ;
					%let cutoff_boolean = ;
					%macro wrapper();

						/*Calculate cut-off variables*/
						%if "&cutoffcat_scale" = "percent" %then %do;
							%let var = percent_overlap;
						%end;
						%else %do;
							%let var = total_days_overlap;
						%end;

						%let temp_cutoffcat = ;
						%let temp_cutoffcat = %sysfunc(tranwrd(&cutoffcat., -, _));
						%let temp_cutoffcat = %sysfunc(tranwrd(&temp_cutoffcat., >, gt));
						%let temp_cutoffcat = %sysfunc(tranwrd(&temp_cutoffcat., <, lt));
						%let temp_cutoffcat = %sysfunc(tranwrd(&temp_cutoffcat., =, e));
						%let temp_cutoffcat = %sysfunc(tranwrd(&temp_cutoffcat., lte, le));
						%let temp_cutoffcat = %sysfunc(tranwrd(&temp_cutoffcat., gte, ge));
							
						%let num_cutoffcat = %sysfunc(countw(&cutoffcat, ' '));
						%do loop = 1 %to &num_cutoffcat;
							/*variable list*/
							%let cutoff_varlist = &cutoff_varlist. %sysfunc(cats(_,%scan(&temp_cutoffcat., &loop.)));
							/*mathematical expressions*/
							%let cat = %scan(&cutoffcat., &loop., ' ');
							%put &cat.;
							/*no range of values (i.e. <10, <=20, >60 >=75)*/
							%if %index(&cat., -) = 0 %then %do;
								%let cat = &var.&cat.;
							%end;
							/*range of values inclusive of boundry (i.e. 25-50)*/
							%else %if %index(&cat., -) > 0 & %index(&cat., <) = 0 & %index(&cat., >) = 0 %then %do;
								%let cat = %sysfunc(tranwrd(&cat., -, <=&var.<=));
							%end;
							/*range of values inclusive of upper boundry, (i.e. 25<-60)*/
							%else %if %index(&cat., -) > 0 & %index(%scan(&cat., 1, '-'), <) > 0 %then %do;
								%let cat = %scan(&cat., 1, '-')&var.<=%scan(&cat., 2, '-');
							%end;
							/*range of values inclusive of lower bountry, (i.e. 50-<75)*/
							%else %if %index(&cat., -) > 0 & %index(%scan(&cat., 2, '-'), <) > 0 %then %do;
								%let cat = %scan(&cat., 1, '-')<=&var.%scan(&cat., 2, '-');
							%end;
								%let cutoff_boolean = &cutoff_boolean. &cat.;
						%end;
						%put &cutoffcat &cutoff_varlist. &cutoff_boolean.;
					%mend;
					%wrapper();

					data _oneperprimary;
						set _oneperprimary;

							if total_days_overlap > 0 then percent_overlap = 100*(total_days_overlap/max_overlap) ;
							else if total_days_overlap = 0 then percent_overlap = 0;

						/*Loop through &cutoff_varlist. and create 0/1 var for each criteria*/
						%do i = 1 %to &num_cutoffcat.;
							%scan(&cutoff_varlist., &i., ' ') = %scan(&cutoff_boolean., &i., ' ');
						%end;
					run;
			
				%end; /*Overlap Episode Analysis*/

				/*Determine whether each primary episode meets adherence*/
				%if %eval(&adherence.=1) %then %do;
					proc sql noprint;
						select count(*) into: num_adherence
						from adherencefile
						where upcase(ANALYSISGRP) = upcase("&ANALYSISGRP.");
					quit;

					%if %eval(&num_adherence > 0) %then %do;
					%let calculate_adherence = Y;

					/*Loop through adherenceID*/
					%do ad = 1 %to &num_adherence.;
							
						/*set macro vars for adherence criteria*/
						data _null_;
							set adherencefile;
							where upcase(ANALYSISGRP) = upcase("&ANALYSISGRP.") and adherenceID = &ad.;
								%if %eval(&multievent.=1) %then %do;
									call symputx("episodelength_start", episodelength_start);
										if episodelength_end ne . then call symputx("episodelength_end", episodelength_end);
										else call symputx("episodelength_end", 99999999); /*Dummy for max window when missing*/
									call symputx("epicount_start", epicount_start);
										if epicount_end ne . then call symputx("epicount_end", epicount_end);
										else call symputx("epicount_end", 99999999);
									call symputx("epigap_start", epigap_start); 
										if epigap_end ne . then call symputx("epigap_end", epigap_end);
										else call symputx("epigap_end", 99999999);
									call symputx("ttepi_start", ttepi_start);
										if ttepi_end ne . then call symputx("ttepi_end", ttepi_end);
										else call symputx("ttepi_end", 99999999);
								%end;
								%if %eval(&overlap.=1) %then %do;
									call symputx("overlap_scale", lowcase(overlap_scale));
									call symputx("overlap_start", overlap_start);
									if overlap_end ne . then call symputx("overlap_end", overlap_end);
										else call symputx("overlap_end", 99999999);
								%end;
						run;

						data _oneperprimary;
							set _oneperprimary;
								
							%if %eval(&multievent.=1) %then %do;
								/*1. Episode Length (tte)*/
								if &episodelength_start. <= tte <= &episodelength_end. then crit_&ad._1 = 1;

								/*2. Number of secondary episodes per primary episode*/
								if &epicount_start. <= epi_count <= &epicount_end. then crit_&ad._2 = 1;

								/*3. Gap between each secondary episode*/
								if min_epigap ne . and min_epigap >= &epigap_start. and max_epigap <= &epigap_end. then crit_&ad._3 = 1;
								else if min_epigap = . then crit_&ad._3 = 1;

									/*Adjust for time_to_censor*/
									if time_to_censor ne . and crit_&ad._3 = 1 then do;
										if (&epigap_start. <= time_to_censor <= &epigap_end.) = 0 then crit_&ad._3 = .;
									end;

								/*4. Time to secondary episode*/
								if &ttepi_start. <= time_to_epi <= &ttepi_end. then crit_&ad._4 = 1;

								if sum(crit_&ad._1, crit_&ad._2, crit_&ad._3, crit_&ad._4) = 4 then ADHERENCE_&ad. = 1;
									else ADHERENCE_&ad. = 0;
							%end;
							%if %eval(&overlap.=1) %then %do;
								%if "&overlap_scale" = "percent" %then %do;
								if &overlap_start. <= percent_overlap <= &overlap_end. then ADHERENCE_&ad. = 1;
									else ADHERENCE_&ad. = 0;
								%end;
								%else %do;
								if &overlap_start. <= total_days_overlap <= &overlap_end. then ADHERENCE_&ad. = 1;
									else ADHERENCE_&ad. = 0;
								%end;
							%end;
						run;

					%end; /*loop through adherenceIDs*/

					/*calculate total adherence and adherence per patients (based on MINADHERENCE / MINADHERENCE_SCALE parameters*/
					data _adherence;
						set _oneperprimary;
						%if %eval(&num_adherence.=1) %then %do;
							ADHERENCE = ADHERENCE_1;
						%end;
						%else %do;
							ADHERENCE = max(of ADHERENCE_:);
						%end;
					run;

					proc means data=_adherence noprint nway missing;
						var ADHERENCE primaryep_num;
						class patid;
						output out=_adherence_per_patid(drop=_:) sum(ADHERENCE) = ADHERENCE_CNT max(primaryep_num)=num_episodes;
					run;

					data _adherence_per_patid; 
						set _adherence_per_patid;

						%if "&MINADHERENCE_SCALE." = "percent" %then %do;
							adherence_pct = 100*(ADHERENCE_CNT/num_episodes);
							ADHERENCE_PAT = (adherence_pct>=&MINADHERENCE.);
						%end;
						%else %do;
							ADHERENCE_PAT = (ADHERENCE_CNT>=&MINADHERENCE.);
						%end;
					run;
					%end;
				%end; /*calculate adherence*/
			%mend createepisodes;

			%createepisodes(truncatedate=);

				/*Merge to create final &runid._mstr_overlap*/
				%if %eval(&multievent.=1) %then %do;
					proc sql noprint;
						create table _mstr_multevent as
						select distinct "&analysisgrp." as analysisgrp length=40
							, mstr.patid
							, mstr.primaryep_num
							, mstr.indexdt
							, mstr.episodeenddt
							, mstr.enr_start
							, mstr.enr_end
							, mstr.year
							, mstr.rawdisp
							, mstr.adjusteddisp
							, mstr.totrxsup
							, mstr.totrxamt
							, mstr.episodetype
							, mstr.tte
							, mstr.tte_cat
							, mstr.tte_catnum
							, mstr.age
							, mstr.birth_date
							, mstr.zip3
							, mstr.state
							, mstr.hhs_reg
							, mstr.cb_reg
							, mstr.zip_uncertain
							, mstr.sex
							, mstr.race
							, mstr.hispanic
							, mstr.agegroup
							, mstr.agegroupnum
							, mstr.minagedate
							, mstr.maxagedate
							, mstr.minepisdur
							, mstr.LastLookFolloweddt
							, mstr.IndexDtLookEndDt
							, mstr.IndexLook
							, month(mstr.indexdt) as month
							, epi.epi_count
							, epi.time_to_epi
							, epi.min_epigap
							, epi.max_epigap
							, epi.epi_gap1
							, epi.epi_gap2
							%if "&calculate_adherence" = "Y" %then %do;
								%do a = 1 %to &num_adherence.;
									, ad.adherence_&a.
								%end;
								, ad.adherence
								, ad1.adherence_pat
							%end;
							%else %do;
								%do a = 1 %to &num_adherence.;
								, 0 as adherence_&a. 
								%end;
								, 0 as adherence
								, 0 as adherence_pat
							%end;
							/*Assign intervention period*/
							%if &maxdates = 2 %then %do;
								,	case when mstr.indexdt < &interventiondate1. then '1'
									when &interventiondate1. <= mstr.indexdt < &interventiondate2. then '2'
									else '3'
									end as interventionperiod
							%end;
							%else %if &maxdates = 1 %then %do;
								,	case when mstr.indexdt < &interventiondate1. then '1'
									else '2'
									end as interventionperiod
							%end;
							%else %do;
							, '0' as interventionperiod %end;
						from primary_episodes as mstr
						left join _oneperprimary as epi on mstr.patid = epi.patid and mstr.primaryep_num = epi.primaryep_num
						%if "&calculate_adherence" = "Y" %then %do;
						left join _adherence as ad on mstr.patid = ad.patid and mstr.primaryep_num = ad.primaryep_num
						left join _adherence_per_patid as ad1 on mstr.patid = ad1.patid
						%end;
						;
					quit;
				%end;
				%if %eval(&overlap.=1) %then %do;
					proc sql noprint;
						create table _mstr_overlap as
						select distinct "&analysisgrp." as analysisgrp length=40
							, mstr.patid
							, mstr.primaryep_num
							, mstr.indexdt
							, mstr.episodeenddt
							, mstr.enr_start
							, mstr.enr_end
							, mstr.year
							, mstr.adjusteddisp as adjusteddisp1
							, mstr.rawdisp as rawdisp1
							, case when sec.adjusteddisp2 > 0 then sec.adjusteddisp2
								else 0
								end as adjusteddisp2
							, case when sec.rawdisp2 > 0 then sec.rawdisp2
								else 0
								end as rawdisp2
							, mstr.totrxsup as totrxsup1
							, case when sec.totrxsup2 > 0 then sec.totrxsup2
								else 0
								end as totrxsup2
							, mstr.totrxamt as totrxamt1
							, case when sec.totrxamt2 > 0 then sec.totrxamt2
								else 0
								end as totrxamt2
							, mstr.episodetype
							, mstr.tte
							, mstr.tte_cat
							, mstr.tte_catnum
							, case when sec.tte2 > 0 then sec.tte2
								else 0
								end as tte2
							, mstr.age
							, mstr.birth_date
							, mstr.sex
							, mstr.race
							, mstr.hispanic
							, mstr.zip3
							, mstr.state
							, mstr.hhs_reg
							, mstr.cb_reg
							, mstr.zip_uncertain
							, mstr.agegroup
							, mstr.agegroupnum
							, mstr.minagedate
							, mstr.maxagedate
							, mstr.LastLookFolloweddt
							, mstr.IndexDtLookEndDt
							, mstr.IndexLook
							, month(mstr.indexdt) as month
							, mstr.minepisdur
							, mstr.obs_start
							, mstr.obs_end
							, sec.epi_count
							, sec.total_days_overlap
							, case when sec.total_days_overlap > 0 then 0
								else 1
								end as zero_overlap
							%do b = 1 %to &num_cutoffcat.;
							, sec.%scan(&cutoff_varlist., &b., ' ')
							%end;
							%if "&calculate_adherence" = "Y" %then %do;
							%do a = 1 %to &num_adherence.;
							, ad.adherence_&a.
							%end;
							, ad.adherence
							, ad1.adherence_pat
							%end;
							%else %do;
							%do a = 1 %to &num_adherence.;
							, 0 as adherence_&a. 
							%end;
							, 0 as adherence
							, 0 as adherence_pat
							%end;
							/*Assign intervention period*/
							%if &maxdates = 2 %then %do;
								,	case when mstr.indexdt < &interventiondate1. then '1'
									when &interventiondate1. <= mstr.indexdt < &interventiondate2. then '2'
									else '3'
									end as interventionperiod
							%end;
							%else %if &maxdates = 1 %then %do;
								,	case when mstr.indexdt < &interventiondate1. then '1'
									else '2'
									end as interventionperiod
							%end;
							%else %do;
							, '0' as interventionperiod %end;
						from primary_episodes as mstr
						left join _oneperprimary as sec on mstr.patid = sec.patid and mstr.primaryep_num = sec.primaryep_num
						%if "&calculate_adherence" = "Y" %then %do;
						left join _adherence as ad on mstr.patid = ad.patid and mstr.primaryep_num = ad.primaryep_num
						left join _adherence_per_patid as ad1 on mstr.patid = ad1.patid
						%end;
						;
					quit;
				%end;
				
			proc datasets nowarn noprint lib=work;
				delete _adherence: _oneperprimary;
			quit;

			/*Create dataset for later output of distribution of epi_gap*/
			%if %eval(&multievent.=1) %then %do;

				*Calculate epi_gap distribution for:
					1. All secondary episodes
					2. Where epi_num = 2
					3. Where epi_num = 3;

				data calculate_epigap;
					set _primary_secondary_eps(where=(epi_gap ne .));
					format analysisgrp $40.;
					count1 = 1;
					if epi_num = 2 then count2 = 1; else count2 = 0;
					if epi_num = 3 then count3 = 1; else count3 = 0;

					year=year(indexdt);
					analysisgrp = "&analysisgrp.";
				run;

				proc datasets nowarn noprint lib=work;
					delete _primary_secondary_eps;
				quit;
			%end;

			%if "&calculate_adherence" = "Y" %then %do;
				%createepisodes(truncatedate=Y);


				%if %eval(&multievent.=1) %then %do;
					proc sql noprint undo_policy=none;
						create table _mstr_multevent as
						select distinct mstr.*
							, pri.indenominator
							, epi.adherence as adherence_truncated
							%do a = 1 %to &num_adherence.;
							, epi.adherence_&a. as adherence_&a._truncated
							%end;
						from _mstr_multevent as mstr
						left join _adherence as epi on mstr.patid = epi.patid and mstr.primaryep_num = epi.primaryep_num
						left join primary_episodes as pri on mstr.patid = pri.patid and mstr.primaryep_num = pri.primaryep_num;
					quit;
				%end;

				%if %eval(&overlap.=1) %then %do;
					proc sql noprint undo_policy=none;
						create table _mstr_overlap as
						select distinct mstr.*
							, pri.indenominator
							, epi.adherence as adherence_truncated
							%do a = 1 %to &num_adherence.;
							, epi.adherence_&a. as adherence_&a._truncated
							%end;
						from _mstr_overlap as mstr
						left join _adherence as epi on mstr.patid = epi.patid and mstr.primaryep_num = epi.primaryep_num
						left join primary_episodes as pri on mstr.patid = pri.patid and mstr.primaryep_num = pri.primaryep_num;
					quit;
				%end;
			%end;
			%else %do;
				%if %eval(&multievent.=1) %then %do;
					proc sql noprint undo_policy=none;
						create table _mstr_multevent as
						select distinct mstr.*
							, pri.indenominator
							, 0 as adherence_truncated
						from _mstr_multevent as mstr
						left join primary_episodes as pri on mstr.patid = pri.patid and mstr.primaryep_num = pri.primaryep_num;
					quit;
				%end;
				%if %eval(&overlap.=1) %then %do;
					proc sql noprint undo_policy=none;
						create table _mstr_overlap as
						select distinct mstr.*
							, pri.indenominator
							, 0 as adherence_truncated
						from _mstr_overlap as mstr
						left join primary_episodes as pri on mstr.patid = pri.patid and mstr.primaryep_num = pri.primaryep_num;
					quit;
				%end;
			%end;

		/** Output to DPLOCAL **/
		%if %eval(&r.=1) %then %do;
			%if %eval(&multievent.=1) %then %do;
				data dplocal.&runid._mstr_multevent ;
					set _mstr_multevent;
				run;
				data dplocal.&runid._mstr_epigap ;
					set calculate_epigap;
				run;
			%end;
			%if %eval(&overlap.=1) %then %do;
				data dplocal.&runid._mstr_overlap;
					set _mstr_overlap;
				run;
			%end;
						
		%end;
		%else %do;
			%if %eval(&multievent.=1) %then %do;
				proc append base=DPLocal.&RUNID._mstr_multevent 
	                   		data=_mstr_multevent force;
	        	run;
				proc append base=DPLocal.&RUNID._mstr_epigap 
	                   		data=calculate_epigap force;
	        	run;
			%end;
			%if %eval(&overlap.=1) %then %do;
				proc append base=DPLocal.&RUNID._mstr_overlap 
	                   	    data=_mstr_overlap force;  
	        	run;
			%end;		
		%end;

		%ms_stoptimer(timestart=secondaryruntime);
		%ms_outputruntimes(timevar=secondaryruntime, step=%str(Secondary Episode Analysis), group=&analysisgrp., monitoringperiod=);

	%end; /*loop through Analysis Groups*/

%put NOTE: ********END OF MACRO: ms_evalsecondaryepi v1.0********;

%mend ms_evalsecondaryepi;
