****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_treeextraction.sas  
*
* Created (mm/dd/yyyy): 06/30/2019
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: Perform outcome extraction for signal detection analysis
* 	                      
*  Program inputs:   
*	- data = dataset that contains patient IDs, index dates, and outcome windows
*	- grp  = name of variable that defines a reference group (group for type 3 or analysisgrp for types 2/4)
*                                    
*  Program outputs:  
*   - 
*                                                                                                  
*  PARAMETERS:     
*
* 
*  Programming Notes:                                                                                                                                                           
*
*	
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_treeextraction(data= , grp=);
		
	%put =====> MACRO CALLED: ms_treeextraction v1.0;

	%ms_starttimer(timestart=treeruntime);

	/*********************************************************************************************/
	/*1 Tree codes pre-processing                                                                */
	/*********************************************************************************************/

	%ms_treeconvertchildparent();	

	*Merge in ICD10 codes if ICD10ICD9 mapping file exists;
	%isdata(dataset=infolder.&ICD10ICD9MAP.);
	%if %eval(&nobs.>0) %then %do;
		proc sql noprint;
			create table expandedtree2 as
			select compress(x.dx, '.') as orig_dx
			, x.dx_codetype as orig_dx_codetype
			, x.dx_codetype
			, compress(x.dx,'.') as dx
			, x.node
			, x.level
			from expandedtree as x
		union all
		select compresS(y.icd10_code,'.') as orig_dx
			, '10' as orig_dx_codetype
			, '09' as dx_codetype
			, compress(y.icd09_code,'.') as dx
			, z.node 
			, z.level
			from infolder.&ICD10ICD9MAP. as y
			inner join expandedtree as z
			on compress(y.icd09_code, '.') = compress(z.dx, '.');
		quit;
	%end;
	%else %do;
		data expandedtree2;
			set expandedtree;
			dx=upcase(compress(dx,'.'));
			orig_dx=dx;
			orig_dx_codetype=dx_codetype;
		run;
	%end;

	*dedup to avoid the extraction of duplicates;
	proc sort data=expandedtree2 out=dx_treecodes_sorted(rename=orig_dx=code rename=orig_dx_codetype = codetype keep=orig_dx orig_dx_codetype) nodupkey;
		by orig_dx orig_dx_codetype;
	run;

	/*********************************************************************************************/
	/*2 Diagnosis code pre-processing                                                            */
	/*********************************************************************************************/

	/*Extract all outcome codes regardless of caresetting*/
	%ms_extractmeds(datafile=&datalib_dia.,
	                   datacode=DX ,
	                   datacodetype=DX_codetype ,
	                   lookfile=dx_treecodes_sorted, 
	                   outfile=dplocal.tree_codes_extract(drop=provider encounterID));

	/*Process envelop macro*/
	%isdata(dataset=dplocal.tree_codes_extract);
	%if %eval(&nobs.>=1) & %eval(&RUN_ENVELOPE. ne 2) %then %do;	
	%ms_envelope(INFILE=dplocal.tree_codes_extract,		               
	    	     ENCOUNTERINFILE=&datalib_enc.,			
	    	     OUTFILE=dplocal.tree_codes_extract);      	 
	%end;

	/*********************************************************************************************/
	/*3 Output table pre-processing                                                              */
	/*********************************************************************************************/

	/*Select all output levels relevant to t#_tree_analysis*/
	data _t&type._tree_levels
		 _t3wkday_levels;
		 format levelvars $150.; /*defensive b/c additional vars added by the code*/
		set infolder.&userstrata.;

			levelvars = lowcase(levelvars);
			tableid = lowcase(tableID);

			*add agegroupnum to strata that include age group;
			if index(lowcase(levelvars), 'agegroup') > 0 then do;
				levelvars = tranwrd(levelvars, "agegroup", "agegroup agegroupnum");
			end;

			*add year when month is requested;
			if index(lowcase(levelvars), 'month') > 0 & index(lowcase(levelvars), 'year') = 0 then do;
				levelvars = tranwrd(levelvars, "month", "year month");
			end;

			*defensive: convert * to space (i.e. agegroup*sex to agegroup sex);
			levelvars = tranwrd(levelvars, "*", " ");

		if lowcase(tableID) = "t&type.treeanalysis" then do;
			*defensive - add tte, ttc, orig_hoi, orig_hoi_codetype, hoi, hoi_codetype if not specified since that is an automatic strata;
			if index(lowcase(levelvars), 'tte') = 0 then do;
				levelvars = catx(' ',levelvars, "tte");
			end;
			if index(lowcase(levelvars), 'ttc') = 0 then do;
				levelvars = catx(' ',levelvars, "ttc");
			end;
			if findw(lowcase(levelvars), 'hoi') = 0 then do;
				levelvars = catx(' ',levelvars, "hoi");
			end;
			if findw(lowcase(levelvars), 'hoi_codetype') = 0 then do;
				levelvars = catx(' ',levelvars, "hoi_codetype");
			end;
			if findw(lowcase(levelvars), 'orig_hoi') = 0 then do;
				levelvars = catx(' ',levelvars, "orig_hoi");
			end;
			if findw(lowcase(levelvars), 'orig_hoi_codetype') = 0 then do;
				levelvars = catx(' ',levelvars, "orig_hoi_codetype");
			end;

			output _t&type._tree_levels;
		end;

		if lowcase(tableID) = "t3treewkdays" then do;
			*Defensive;
			if findw(lowcase(levelvars), 'hoi') = 0 then do;
				levelvars = catx(' ',levelvars, "hoi");
			end;
			if findw(lowcase(levelvars), 'orig_hoi') = 0 then do;
				levelvars = catx(' ',levelvars, "orig_hoi");
			end;

			if index(lowcase(levelvars), 'wkday') = 0 then do;
				levelvars = catx(' ',levelvars, "wkday");
			end;
			output _t3wkday_levels;
		end;
	run;

	%let treestrat1 = ;
	%let treewkdaystrat1 = ;
	proc sql noprint;
		select distinct lowcase(levelvars) into: treestrat1 separated by ' '
		from _t&type._tree_levels;
		select distinct lowcase(levelvars) into: treewkdaystrat1 separated by ' '
		from _t3wkday_levels;
	quit;
	%nonrep(invar = treestrat1, outvar = treestrat);
	%nonrep(invar = treewkdaystrat1, outvar = treewkdaystrat);

	/*Generic macro to compute MSOC output tables*/
	%macro createNumTableEntries(class=, level=);
		proc means data=tree_final nway noprint missing;
			var h;
			class treeanalysisgrp group &class.;
			output out=_num0(drop=_:) sum(h)=nhois;      
	    run;
			
	    %if %eval(&s.=1) %then %do;
		   	data _hoicounts;
			    set _num0;
			    format level $3.;
			    level="&level.";
			run;
		%end;
		%else %do;
			data _hoicounts;
				set _hoicounts
			        _num0(in=a);
			    if a then do;	
					level="&level.";
				end;
		    run;
	    %end;
		proc datasets nowarn noprint lib=work;
			delete _num0;
		quit;
   %mend; 

 	%ms_stoptimer(timestart=treeruntime);
	%ms_outputruntimes(timevar=treeruntime, step=%str(Tree Extraction Module Pre-Extraction), group=, monitoringperiod=);

	/*********************************************************************************************/
	/*4 Loop through each TREEANALYSISGRP                                                        */
	/*********************************************************************************************/

	%isdata(dataset=infolder.&treefile.); 
	data _null_;
    call symputx("num_order","&nobs.");
    run;
	%put The number of tree analysis groups is &num_order.;

	%do r = 1 %to &num_order.;

		%ms_starttimer(timestart=treegrpruntime);

		/******************************************/
		/* Assign macro variables                 */
		/******************************************/
	
		/*reset macro vars*/
		%let treeanalysisgrp = ;
		%let refgroup = ;
		%let tree_incident_level = ;
		%let treewashper = ;
		%let treecaresettingprincipal = ;
		%let treecaresettingprincipal_washout = ;
		%let extractincidence = N;
		%let grpname=;
		%let grpname2=;
		%let treestartdate = ;
		%let treeenddate = ;
		%let eraspan = N;

		data _null_;
			set infolder.&treefile.;
			if _n_ = &r. then do;
				call symputx("treeanalysisgrp", strip(treeanalysisgrp));
				call symputx("refgroup", strip(group));
				call symputx("tree_incident_level", tree_incident_level);
				call symputx("treewashper", treewashper);
				call symputx("treecaresettingprincipal", treecaresettingprincipal);
				call symputx("treecaresettingprincipal_washout", treecaresettingprincipal_washout);

				*indicator that oucome and incidence caresettingprincipal are the same than only need to extract codes 1x;
				if treecaresettingprincipal ne treecaresettingprincipal_washout then call symputx("extractincidence", "Y");
			end;
		run;
		%put now looping on order: &r.;
		%put treeanalysisgrp: &treeanalysisgrp.;

		/*Additional parameters needed for treeanalysisgrp loop*/

			/*Query Period*/
			%if %eval(&type.=2) | %eval(&type.=4) %then %do;
			data _null_;
				set infolder.&comparison.;
				if analysisgrp = "&refgroup" then do;
					call symputx('grpname', strip(eoi));
					call symputx('grpname2', strip(ref));
				end; 
			run;
			%end;
			%if %eval(&type.=3) %then %do;
				%let grpname = &refgroup.;
			%end;
			%if %eval(&type.=4) %then %do;
				*Original MIL eoi group;
				%let position_ = %sysfunc(find(&grpname., _, -40));
				%let grpname_metadata = %substr(&grpname, 1, %eval(&position_.-1));
			%end;

			data _null_;
				set msoc.&runid._metadata_for_time_period_&periodidstart.(where=(timeperiod=&periodidstart.));
				%if %eval(&type.=2) | %eval(&type.=3) %then %do;
				if group = "&grpname." then do;
				%end;
				%if %eval(&type.=4) %then %do;
				if group = "&grpname_metadata." then do;
				%end;
					call symputx('treestartdate', startdate);
					call symputx('treeenddate', survenddate);

					*Determine is query period spans ICD9-ICD10 era;
					if startdate <= '30Sep2015'd and survenddate >= '30Sep2015'd then call symputx('eraspan', 'Y');
				end; 
			run;

			/*Age stratification*/
			data _null_;
				set infolder.&COHORTFILE.;
				if strip(cohortgrp) = "&grpname." then do;
					*Age stratification;
					if missing(agestrat) then do;
						call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
					end;
					else do;
						call symputx("agestrat",agestrat);
					end;
				end;
			run;


		/******************************************/
		/* Outcome extraction processing          */
		/******************************************/

		*Merge in node for &tree_incident_level and create caresettingprincipal variables in lookup table;
		data treetable_loop(rename=orig_dx=code rename=orig_dx_codetype=codetype);
			set expandedtree2(in=a where=(level=&tree_incident_level.))
			%if "&extractincidence" = "Y" %then %do;
			expandedtree2(in=b where=(level=&tree_incident_level.))
			%end;
			;

			format caresettingprincipal $32. tree_fup $3.;
			if a then do;
				tree_fup = 'DEF';
				caresettingprincipal = "&treecaresettingprincipal";
			end;
			%if "&extractincidence" = "Y" %then %do;
			if b then do;
				tree_fup = 'IOC';
				caresettingprincipal = "&treecaresettingprincipal_washout";
			end;
			%end;
		run;

	    *Break out CaresettingPrincipal into Caresetting and Principal;
	    %ms_caresettingprincipal(InFile=treetable_loop, 
	                             Var=CareSettingPrincipal, 
	                             OutFile=treetable_loop);


		/******************************************/
		/* Extract outcome/incidence codes        */
		/******************************************/

		*Restrict dplocal.tree_codes_extract to pts in relevant group;
		%if %eval(&type.=2) %then %do;
		data ptslist;
			set &data.;
			if &grp. = "&refgroup";
			rename studyclass=group;
			keep &grp. studyclass patid;
		run;
		%end;
		%if %eval(&type.=3) %then %do;
		data ptslist(keep=patid &grp.);
			set &data.;
			if &grp. = "&refgroup";
		run;
		%end;
		*Type 4 - outcomes in infants;
		%if %eval(&type.=4) %then %do;
		data ptslist(rename=linkedid=patid);
			set &data.;
			if &grp. = "&refgroup";
			rename studyclass=group;
			keep linkedid &grp. studyclass;
		run;
		%end;

		*Check to ensure at least 1 patient in group;
		%isdata(dataset=ptslist);
		%if %eval(&nobs.>0) %then %do;

			data dplocal.tree_extract&r.;
				if 0 then set PtsList(keep=patid);
				declare hash pt (hashexp:16, dataset:"PtsList");
				pt.definekey('PatId');
				pt.definedone();

				do until(eof1);
					set dplocal.tree_codes_extract end=eof1;
						if pt.find()=0 then do;
							output;
						end;
				end;
				stop;
			run;

			*Extract all outcome and incidence codes;
		    %ms_loopmeds(datafile=dplocal.tree_extract&r.,
		                 lookfile=treetable_loop, 
						 multiplecodecat = N,
		                 outfile=dplocal._treeloop_diags);

			*Assign group name;
			data dplocal.tree_extract_withgroup&r.;
				if 0 then set PtsList;
				declare hash pt (hashexp:16, dataset:"PtsList");
				pt.definekey('PatId');
				%if %eval(&type.=3) %then %do;
				pt.definedata("&grp.");
				%end;
				%else %do;
				pt.definedata("&grp.", "group");
				%end;
				pt.definedone();

				do until(eof1);
					set dplocal._treeloop_diags end=eof1;
						if pt.find()=0 then do;
							output;
						end;
				end;
				stop;
			run;

			*Split into DEF and IOC codes;
			data _treefupcodes
				 _treewashcodes;
				 set dplocal.tree_extract_withgroup&r.;

				 if tree_fup = 'DEF' then output _treefupcodes;
				 if tree_fup = 'IOC' then output _treewashcodes;
			run;

	    	%if "&extractincidence" = "N" %then %do;
				data _treewashcodes;
					set _treefupcodes;
					tree_fup = 'IOC';
				run;
			%end;

			/******************************************/
			/* Find all potential outcomes		      */
			/******************************************/

			/*For Types 2 and 4, discard uninformative followup (i.e. censor at the end of followup of the matched set)*/
			%if %eval(&type.=2) | %eval(&type.=4) %then %do;
				data ptslist;
					set &data.;
					if &grp. = "&refgroup";

					exp=0;
					unexp=0;
					FUTimeExp=0;
					FUTimeUnExp=0;

					if exposure=1 then do;
						Exp=1;
						FUTimeExp=Followuptime;
					end;
					if exposure=0 then do;
						UnExp=1;
						FUTimeUnExp=Followuptime;
				   	end;

					%if %eval(&type.=2) %then %do;
						treeoutcomestart = indexdt + t2atriskstart;
					%end;
					%if %eval(&type.=4) %then %do;
						treeoutcomestart = outcomestart;
					%end;

					keep patid exp unexp FUTimeExp FUTimeUnExp matchid indexdt studyclass treeoutcomestart ttc followuptime
					age_cat agegroupnum cb_reg hhs_reg hispanic month race sex state year zip3 zip_uncertain 
					%if %eval(&type.=4) %then %do; linkedid %end;
					%if %eval(&NumCovars.>0) %then %do; covar: %end;;
				run;

			    *find maximum followuptime in each arm for each matchid;
			    proc means data=ptslist nway noprint;
				    var FUTimeExp FUTimeUnExp;
				    class MatchID ;
				    output out=max_fu_time(drop=_:) max=maxFUTimeExp maxFUTimeUnExp;
			    run;

			    proc sql noprint undo_policy=none;
			    	create table ptslist as
				    select dat.*,
				           min(fu.maxFUTimeExp,fu.maxFUTimeUnExp) as stopFU
				    from ptslist as dat left join
				         max_fu_time as fu
				    on dat.matchID = fu.matchID;
			    quit; 

				*if type = 4 - only keep linkedID;
			    data ptslist(rename=studyclass = group rename=age_cat=agegroup %if %eval(&type.=4) %then %do;rename=linkedid = patid %end;);
				    set ptslist;
				    if Followuptime > stopfu then do;
				      Followuptime= stopfu;
				    end;

					treeoutcomeend = Indexdt+followuptime - 1;
					format treeoutcomestart treeoutcomeend date9.;

					keep patid indexdt studyclass treeoutcomestart treeoutcomeend ttc
					age_cat agegroupnum cb_reg hhs_reg hispanic month race sex state year zip3 zip_uncertain 
					%if %eval(&type.=4) %then %do; linkedid %end;
					%if %eval(&NumCovars.>0) %then %do; covar: %end;;
					%if %eval(&type. = 4) %then %do; rename patid=origpatid ;%end;
			    run;
			%end;
			%if %eval(&type.=3) %then %do;
				data ptslist;
					set &data.;
					where &grp. = "&refgroup";

					treeoutcomestart = T3RiskFromDt;
					treeoutcomeend = T3RiskToDt;
					format treeoutcomestart treeoutcomeend date9.;

					keep patid indexdt group treeoutcomestart treeoutcomeend ttc 
						agegroup agegroupnum cb_reg hhs_reg hispanic month race sex state year zip3 zip_uncertain 
						%if %eval(&NumCovars.>0) %then %do; covar: %end;;
				run;
			%end;

			/*Extract outcomes*/
			proc sql noprint;
				create table tree_outcomes as
				select distinct pts.patid,
					   pts.group,
					   pts.indexdt,
					   out.code as orig_hoi,
					   out.codetype as orig_hoi_codetype,
					   out.adate as hoidate,
					   out.node
				from ptslist as pts
				inner join _treefupcodes as out
				on pts.patid = out.patid
				where pts.treeoutcomestart <= out.adate <=pts.treeoutcomeend
				order by group, patid, indexdt, hoidate, node;
			quit;

			/*Tie breaker procedure if two outcomes occur on the same day in the same node*/
				*patid, indexdt, hoidate, and node of all ties;
				proc sql noprint;
					create table ties as
					select distinct patid, 
						   indexdt,
						   hoidate,
						   node,
						   count(*) as count length=3
					from tree_outcomes
					group by patid, hoidate, node;
				quit;

				*codes to evaluate tiebreaker;
				proc sql noprint;
					create table tiecodes as
					select distinct group, 
							        orig_hoi as code, 
								    orig_hoi_codetype as codetype
					from tree_outcomes as x,
					ties as y
					where x.patid = y.patid and x.hoidate = y.hoidate and x.node=y.node and y.count>1;
				quit;

				%isdata(dataset=tiecodes);
				%if %eval(&nobs.>0) %then %do;

					*Find overall occurence in exposure cohort over the corse of the entire query period;
					data dplocal.tielist;
						if 0 then set tiecodes;
						declare hash pt (hashexp:16, dataset:"tiecodes");
						pt.definekey("group",'code','codetype');
						pt.definedone();

						do until(eof1);
							set _treefupcodes end=eof1;
								if pt.find()=0 then do;
									if &treestartdate. <= adate <= &treeenddate. then output;
								end;
						end;
						stop;
					run;

					proc sql noprint;
						create table dplocal.tiecount as 
						select group,
							   code,
							   codetype,
							   count(*) as freq
						from dplocal.tielist
						group by group, code, codetype;
					quit;

					*Separate out hoidates with ties vs no ties;
					proc sql noprint;
						create table ties_outcomes as
						select x.*, 
							   c.freq
						from tree_outcomes as x
						inner join ties as y
							on x.patid = y.patid and x.indexdt = y.indexdt and x.hoidate = y.hoidate and x.node = y.node
						inner join dplocal.tiecount as c
							on x.group = c.group and x.orig_hoi = c.code and x.orig_hoi_codetype = c.codetype
						where y.count >1;

						create table noties_outcomes as 
						select x.*
						from tree_outcomes as x
						inner join ties as y
						on x.patid = y.patid and x.indexdt = y.indexdt and x.hoidate = y.hoidate and x.node = y.node
						where y.count=1;
					quit;

					*Incase of a tie in frequency, assign a random number;
					data ties_outcomes;
						set ties_outcomes;
						call streaminit(1234567);
						ToRandSelect = rand('uniform');
					run;

					proc sort data=ties_outcomes out=ties_outcomes;
						by group patid indexdt hoidate node freq torandselect;
					run;

					data ties_outcomes1;
						set ties_outcomes;
						by group patid indexdt hoidate node;
						if first.node;
					run;

					proc sql noprint;
						create table tree_outcomes as
						select * from ties_outcomes1(drop=freq torandselect)
						union all
						select * from noties_outcomes
						order by group, patid, indexdt, hoidate, node;
					quit;

				%end; /*ties*/

			/***************************************************/
			/* Find all potential outcomes that meet incidence */
			/***************************************************/

			*List of valid outcome dates;
			proc sql noprint;
				create table ptsoutcomes as
				select group, patid, indexdt, hoidate, node
				from tree_outcomes;
			quit;

			*Extract incident outcomes;
			proc sql noprint;
				create table incident_outcomes as
				select distinct pts.patid,
					   pts.group,
					   pts.indexdt,
					   pts.hoidate,
					   pts.node
				from tree_outcomes as pts
				inner join _treewashcodes as inc
				on pts.patid = inc.patid and pts.node = inc.node
				where %ms_periodsoverlap(period1=pts.hoidate-min(&treewashper.,99999) pts.hoidate-1,
	                                       period2=Inc.adate)
				order by group, patid, indexdt, hoidate, node;
			quit;

			*Discard outcomes if not incident;
			data tree_final_outcomes;
				merge tree_outcomes(in=a) incident_outcomes(in=b);
				by group patid indexdt hoidate node;
				if a and not b;
			run;

			/*********************************************/
			/* Compute all variables and save to DPLOCAL */
			/*********************************************/

			*Merge in mapped codes, TTE, wkday;
			%isdata(dataset=tree_final_outcomes);
			%if %eval(&nobs.>0) %then %do;
				proc sql noprint;
					create table tree_final%if %eval(&type.=4) %then %do;(rename=patid=linkedid rename=origpatid=patid)%end; as
					select "&treeanalysisgrp" as treeanalysisgrp length=40 format=$40.,
						   pts.*,
						   outcome.orig_hoi,
					       outcome.orig_hoi_codetype,
						   outcome.hoidate,
						   outcome.node,
						   outcome.hoidate - pts.indexdt as tte length=3,
						   %if %eval(&type.=3) %then %do;
						   weekday(outcome.hoidate) as wkday length=3,
						   %end;
						   %if "&eraspan" = "Y" %then %do;
						   map.dx as hoi,
						   map.dx_codetype as hoi_codetype,
						   %end;
						   %else %do;
						   outcome.orig_hoi as hoi,
					       outcome.orig_hoi_codetype as hoi_codetype,
						   %end;
						   1 as h /*for creating output tables*/
					from ptslist as pts
					inner join tree_final_outcomes as outcome
						on pts.patid = outcome.patid and pts.group=outcome.group and pts.indexdt=outcome.indexdt
					left join expandedtree2(where=(level=&tree_incident_level.)) as map 
						on outcome.orig_hoi = map.orig_dx and outcome.orig_hoi_codetype = map.orig_dx_codetype;
				quit;

			%end;
			%else %do;
				data tree_final;
					set ptslist(obs=0) expandedtree2(obs=0 keep=orig_dx orig_dx_codetype node);

					orig_hoi = orig_dx;
					orig_hoi_codetype = orig_dx_codetype;
					hoi = orig_dx;
					hoi_codetype = orig_dx_codetype;

					format hoidate date9. treeanalysisgrp $40.;
					length tte wkday 3 treeanalysisgrp $40;
					treeanalysisgrp = "&treeanalysisgrp.";
					tte=.;
					wkday = .;
					hoidate= .;
					h=.;

					drop orig_dx orig_dx_codetype;
				run;
			%end;

			*Clean up;
			proc datasets nowarn noprint lib=dplocal;
				delete tie: _treeloop_diags incident_outcomes tree_extract:;
			quit;

			proc datasets nowarn noprint lib=work;
				delete ptslist treetable_loop tree_outcomes _treefupcodes _treeioccodes
					   ties tiecodes noties incident_outcomes ptsoutcomes tree_final_outcomes;
			quit;

			/********************************/
			/* Output tables to MSOC folder */
			/********************************/

			/************************************************/
			/* [runid]_t#_tree_analysis_[periodid].sas7bdat */
			/************************************************/

			%isdata(dataset=_t&type._tree_levels);
			%if %eval(&nobs.>0) %then %do;
				
				%do s = 1 %to %eval(&nobs.);
					data _null_;
						set _t&type._tree_levels;
						if _n_ = &s. then do;
							call symputx('levelid', levelid);
							call symputx('levelvars', levelvars);
						end;
					run;

					*Note: these tables will not be squared;

					/*Summarize data */
					%createNumTableEntries(class=&levelvars., level=&levelid.);

					proc datasets nowarn noprint lib=work;
						delete _mstrtree _mastersquare;
					quit;

				%end;

				proc sort data=_hoicounts;
					by treeanalysisgrp group level &treestrat.;
				run;

				%if %eval(&r.=1) | %sysfunc(exist(msoc.&runid._t&type._tree_analysis_&periodidstart))=0 %then %do;
					/*Save to MSOC*/
					data msoc.&runid._t&type._tree_analysis_&periodidstart.; 
						retain treeanalysisgrp group level &treestrat. nhois;
						set _hoicounts;
					run;
				%end;
				%else %do;
					proc append base=msoc.&runid._t&type._tree_analysis_&periodidstart.
						data=_hoicounts(keep=treeanalysisgrp group level &treestrat. nhois) force;
					run;
				%end;

				proc datasets nowarn noprint lib=work;
					delete _hoicounts;
				quit;

			%end;

			/**********************************************/
			/* [runid]_t3_tree_wkdays_[periodid].sas7bdat */
			/**********************************************/

			%if %eval(&type.=3) %then %do;

				%isdata(dataset=_t3wkday_levels);
				%if %eval(&nobs.>0) %then %do;

					%do s = 1 %to %eval(&nobs.);
						data _null_;
							set _t3wkday_levels;
							if _n_ = &s. then do;
								call symputx('levelid', levelid);
								call symputx('levelvars', levelvars);
							end;
						run;

						*Note: these tables will not be squared;

						/*Summarize data */
						%createNumTableEntries(class=&levelvars., level=&levelid.);

						proc datasets nowarn noprint lib=work;
							delete _mstrtree _mastersquare;
						quit;

					%end;

					proc sort data=_hoicounts;
						by treeanalysisgrp group level &treewkdaystrat.;
					run;

					%if %eval(&r.=1) | %sysfunc(exist(msoc.&runid._t3_tree_wkdays_&periodidstart.))=0 %then %do;
						/*Save to MSOC*/
						data msoc.&runid._t3_tree_wkdays_&periodidstart.; 
							retain treeanalysisgrp group level &treewkdaystrat. count;
							set _hoicounts(rename=nhois=count);
						run;
					%end;
					%else %do;
						proc append base=msoc.&runid._t3_tree_wkdays_&periodidstart
							data=_hoicounts(keep=treeanalysisgrp group level &treewkdaystrat. nhois rename=nhois=count) force;
						run;
					%end;

					proc datasets nowarn noprint lib=work;
						delete _hoicounts;
					quit;
				%end;
			%end;

			/**************************************************************/
			/*Update attrition table with # of patients with an outcome   */
			/**************************************************************/

			%if %eval(&type.=3) %then %do;

				%let hoipatients = 0;
				proc sql noprint;
					select count(distinct patid) into: hoipatients
					from tree_final;
				quit;

				data msoc.&runid._attrition;
					set msoc.&runid._attrition end=eof;
					output;
					if eof then do;
						group = "&refgroup.";
						level = 99;
						descr = "Information: Members with at least one HOI in a HOI assessment window for tree extraction analysis: &treeanalysisgrp";
						remaining = &hoipatients.;
						excluded = .;
						output;
					end;
				run;

			%end;
			%if %eval(&type.=2) | %eval(&type.=4) %then %do;
				
			%let hoipatients_eoi = 0;
			%let hoipatients_ref = 0;
				proc sql noprint;
					select count(distinct patid) into: hoipatients_eoi
					from tree_final
					where group = "&grpname";
					select count(distinct patid) into: hoipatients_ref
					from tree_final
					where group = "&grpname2";
				quit;

				data msoc.&runid._match_attrition_&periodidstart.;
					set msoc.&runid._match_attrition_&periodidstart. end=eof;
					output;
					if eof then do;
						analysisgrp = "&refgroup";
						eoi = "&grpname.";
						ref = "&grpname2.";
						level = 99;
						descr = "Information: Members with at least one HOI in a HOI assessment window for tree extraction analysis: &treeanalysisgrp";
						eoi_remaining = &hoipatients_eoi.;
						eoi_excluded = .;
						ref_remaining = &hoipatients_ref.;
						ref_excluded = .;
						output;
					end;
				run;

			%end;

			*Save patient level data to DPLOCAL;
			%if %eval(&r.=1) | %sysfunc(exist(dplocal.&runid._mstr_tree))=0 %then %do;
				data dplocal.&runid._mstr_tree;
					set tree_final(drop=h);
				run;
			%end;
			%else %do;
	 			data dplocal.&runid._mstr_tree;
					set dplocal.&runid._mstr_tree
					    tree_final(drop=h);
				run;
			%end;

		%end; /*at least 1 patient in group*/

		proc datasets nowarn noprint lib=work;
			delete tree_:
		quit;

		%ms_stoptimer(timestart=treegrpruntime);
		%ms_outputruntimes(timevar=treegrpruntime, step=%str(Tree Extraction Module), group=&treeanalysisgrp, monitoringperiod=);

	%end; /*treeanalysisgrp loop*/

	proc datasets nowarn noprint lib=work;
		delete  _t:;
	quit;
	proc datasets nowarn noprint lib=dplocal;
		delete tree_codes_extract
	quit;

	%put NOTE: ******** END OF MACRO: ms_treeextraction v1.0 ********;

%mend ms_treeextraction;
