****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_t1t2_createreport.sas  
*
* Created (mm/dd/yyyy): 12/27/2017
* Last modified: 12/27/2018
* Version: 1.3
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: This macro creates each dataset necessary to produce a T1/T2 report
*                                        
*  Program inputs:                                                                                   
*
* 
*  Program outputs:                                                                                                                                       
* 
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.1      03/26/18     RR		Added row/s for Never exposed cohort (if applicable) (QRP-554)
*
*    1.2      07/01/18     AP		Modified for QRP 6.0.0 compatibility
*
*    1.3      12/27/18     AP		Changed macro name from ms_create_t1t2_report to ms_t1t2_createreport
*
***************************************************************************************************;

%macro ms_t1t2_createreport();

/**************************************************************************************************
*	1. Write log to external file, Clean up work, assign libnames and default macros vars 															
***************************************************************************************************/

	%put =====> MACRO CALLED: ms_t1t2_createreport v1.3;

	proc datasets nowarn nolist noprint lib=work kill; quit;

	/*global macro variables*/
 	%ms_t1t2_initializemacrovariables();

 	/*Define all groups and runids*/
 	%ms_t1t2_definegroupsruns();

	/*Assign default cellpadding value*/
	%if %length(&line_spacing.) = 0 %then %do;
		%let line_spacing = 1.75;
	%end;

	/*stack all the cohortfiles, there may be multiple cohortfiles for multiple runs*/
	data cohortfile;
		set 
		%do j = 1 %to %sysfunc(countw(&cohortfile.));
			infolder.%scan(&cohortfile., &j.)
		%end;;
	run;

  /*stack all the typefiles, there may be multiple typefiles for multiple runs*/
	data typefiles;
		set 
		%do j = 1 %to %sysfunc(countw(&alltypefiles.));
			infolder.%scan(&alltypefiles., &j.)
		%end;;
	run;

	%IF &CREATEREPORT_TYPE. = 2 %THEN %DO;
		 proc sql noprint;
	  	 	create table _cohorttype as
		  	select a.*, b.NEVEREXPOSEDCOHORT
		  	from cohortfile a left join typefiles b
		  	on upcase(strip(a.cohortgrp)) = upcase(strip(b.group));
	   	quit;
	   data cohortfile (drop = NEVEREXPOSEDCOHORT);
			length cohortgrp $40;
			set _cohorttype;
			 output;
			 if upcase(strip(NEVEREXPOSEDCOHORT)) = "Y" then do;
			 	cohortgrp = cats(cohortgrp,"_nvrexp");
				output;
			 end;
	   run;
	%END;

	/*Determine if race/hispanic should be produced*/
	data userstrata;
		format levelvars $150.;
		set infolder.&USERSTRATA.;
		levelvars = lowcase(levelvars);
		tableid = lowcase(tableID);
		levelvars = tranwrd(levelvars, "*", " ");
	run;

	*Master list of all strata;
	proc sql noprint;
		select distinct lowcase(levelvars) into: allstrat1 separated by ' '
		from userstrata;
	quit;

	%nonrep(invar=allstrat1 , outvar=allstrat);
	%put &allstrat.;

	*List of covariates;
	%do c = 1 %to %sysfunc(countw(&allstrat.));
		%if %substr(%scan(&allstrat., &c.),1,3) = cov %then %do;
			%let covarlist = &covarlist. %scan(&allstrat., &c.);
		%end;
	%end;
	%put &covarlist.;
	%let classvars = &classvars. &covarlist.;

	%let race_out = N;
	%let hispanic_out = N;
	%if %index(&allstrat., race) > 0 %then %do;
		%let race_out = Y;
	%end;
	%if %index(&allstrat., hispanic) > 0 %then %do;
		%let hispanic_out = Y;
	%end;

	data appendixA; 
		format DPEndDate DPStartDate $10. DPMINDATE DPMAXDATE date9.;
		DPMINDATE=&DP_MinDate.;
		DPMAXDATE=&DP_MaxDate.;
		DPstartDate = strip(put(DPMINDATE, date9.));
		DPEndDate = strip(put(min("&enddate3."d, DPMAXDATE), date9.));		
	run;

	proc sql noprint; 
	   	create table &REQUESTID._appendixA as
	   	select input(strip(DPstartDate),date9.) as DPStartDate format=mmddyy10., 
				input(strip(DPEndDate),date9.) as DPEndDate format=mmddyy10.,
				"DP01" as maskedID
	    from appendixA;
	quit;

	/*Assign formats*/
	%ms_t1t2_assignformats();

/************************************************************************************************************************************
*	2. Aggregate and calculate baseline tables for level 1 analysis							
************************************************************************************************************************************/

	%if "%upcase(&Output_baselinetable.)" = "Y" %then %do;
		%ms_t1t2_createbaselinetable();
	%end;

/************************************************************************************************************************************
*	4. Aggregate and calculate summary tables					
************************************************************************************************************************************/

	%ms_t1t2table();

/************************************************************************************************************************************
*	4. Aggregate and calculate CDF figures				
************************************************************************************************************************************/

	%if "%upcase(&Output_CDF_KM.)" = "Y" %then %do;
		%ms_t1t2_createcdf();	
	%end;

/************************************************************************************************************************************
*	5. Output report			
************************************************************************************************************************************/

	%ms_t1t2_outputreport();

/*--------------------------------------------------------------------------------------------*/
/* 5: Clean Work        																      */
/*--------------------------------------------------------------------------------------------*/

	proc datasets nowarn nolist lib=work kill; quit;
	
	%put =====> END MACRO: ms_t1t2_createreport v1.3;

%mend ms_t1t2_createreport;
