****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_t5_create_distribution_tables.sas  
*
* Created (mm/dd/yyyy): 10/13/2017
* Last modified: 02/05/2019
* Version: 2.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: The macro buckets a continuous variable into categories and outputs counts and percents
*
*  Program inputs:                                                                                   
*  	-
* 
*  Program outputs:                                                                                                                                       
*  	-
* 
*  PARAMETERS:        

*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  01/23/18   AP			Filled in missing cells with 0/0.0% (QRPL-175)
*
*   1.2		  05/15/18   AP			Expanded 0 cell rows and removed proc univariate freq statement
*									in order to keep squared tables (QRPL-184)
*
*   1.3	      05/18/18   RR			Fix the distribution with value 0 (QRPL-223) 	
*
*   1.4       06/28/18	 RT         Update the macro to create statistics by group 
*                                   added colnvar to indicate which variable to use for N in descriptive table.
*
*   1.5       11/27/18   RT         sub set _distributiondata before creating _distributiondata_exp, squrare totalcategories
*
*   2.0       12/27/18   AP         Recoded for integration into QRP
*
*   2.1       02/05/19   AP         Added additional 2 level stratifications
*
*********************************************************************************************************;

%macro ms_t5_create_distribution_tables(data=, 
									    countvar=,
									    catvar=, 
									    categories =,
									    cattable = ,
									    disttable =);

	%put =====> MACRO CALLED: ms_t5_create_distribution_tables v2.1;

	/*Category and distribution tables take the same levelIDs. For efficiency will extract 1x*/
	%let table = "&cattable.","&disttable.";

	data _tablecheck&cattable.
		 _tablecheck&disttable.;
		 set &tablefile.(where=(table in (&table.)));
		 
		 if table = "&cattable" then output _tablecheck&cattable.;
		 if table = "&disttable" then output _tablecheck&disttable.;
	run;

	/*Set &cattable and &disttable to missing if not requested*/
	%isdata(dataset=_tablecheck&cattable.);
	%if %eval(&nobs.<1) %then %do;
		%let cattable = ;
	%end;
	%isdata(dataset=_tablecheck&disttable.);
	%if %eval(&nobs.<1) %then %do;
		%let disttable = ;
	%end;

	/*Categories*/
	%if "&cattable." ne "" %then %do;
	%convert_strat(var=&catvar., categories=&categories.);
	%end;

	*Compute overall - A table required;
	data _table_A;
		set &tablefile.(where=(table in (&table.) and tablesub='A'));
	run;

	%isdata(dataset=_table_A);
	%if %eval(&nobs.>0) %then %do;
		
		data _null_;
	 		set _table_A;
			if _n_ = 1 then do;
				call symputx('levelid1', levelid1);
				call symputx('levelid2', levelid2);
			end;
		run;

		/*Extract column 1: Total*/
		proc sort data=&data. out=_total_bydp(rename=&countvar.=total_count keep=runid group &countvar.);
			by runid group;
			where level in ("&levelid1.");
		run;

		/*Group continous var into categories*/
		%if "&cattable." ne "" %then %do;
			data _catdata;
				set &data.(where=(level in ("&levelid2.")));

				*assign strata;
				length strat $50.;
				%do c =1 %to &num_categories.;
					if %scan(&categories_boolean., &c., ' ') then do;
						strat = "%scan(&categories., &c., ' ')";
						stratsort = &c.;
					end;
				%end;

				%if %length(&categories)>0 %then %do;
				  if stratsort = . then do;
					strat = 'delete';
					stratsort = 999;
				  end;
				%end;
				%else %do;
				  stratsort=1;
				  strat='';
				%end;

				if strat='delete' then delete;
			run;

			*collapse across strata;
			proc means noprint nway data=_catdata;
				var &countvar.;
				class runid group strat stratsort / missing;
				output out=_distribution_cat_a(drop=_:) sum=;
			run;

			*Transpose and square;	
			proc transpose data=_distribution_cat_a out=_catdata_trans(drop=_NAME_) prefix=_;
				by runid group;
				var &countvar.;
				id stratsort;
			run;

			data table_&cattable.a;
				merge _total_bydp _catdata_trans;
				by runid group;
				
				*First ensure all category variables exist;
				if _n_ = 1 then do;
					dsid = open("_catdata_trans");
					%do c =1 %to &num_categories.;
						if varnum(dsid,"_&c.") = 0 then _&c. =.;
					%end;
					rc= close(dsid);
				end;
				drop rc dsid;

				*Compute percent;
				format total_percent 12.1;
				total_percent = 100.0;
				%do c =1 %to &num_categories.;
					if total_count >0 then do;
						_&c._percent = (_&c./total_count)*100;
					end;
				%end;

				*Fill in missing with 0s, Add labels and formats;
				%do c =1 %to &num_categories.;
					if _&c. = . then _&c. = 0;
					if _&c._percent = . then _&c._percent = 0;
					label _&c. = "%scan(&categories., &c., ' ')";
					label _&c._percent = "%scan(&categories., &c., ' ') %";
					format _&c._percent 12.1 _&c. comma12.0; 
				%end;

				format total_count comma12.0;
			run;

			proc datasets nowarn noprint lib=work;
				delete _catdata_trans _catdata _total_bystrat;
			quit;
		%end;

		/*Continous var metrics*/
		%if "&disttable." ne "" %then %do;
			proc means data=&data.(where=(level in ("&levelid2."))) noprint nway;
				var &catvar.;
				freq &countvar.;
				class runid group;
				output out=table_&disttable.a(drop=_type_ _freq_) p25=_p25 p75=_p75 
														max=_max 
														min=_min 
														median=_median 
														mean=_mean 
														std =_std;		
			run;	

			/*merge in squared table - freq statement drops rows where &countvar.=0*/
			proc sort data=&data.(where=(level = "&levelid2")) nodupkey out=table_&disttable.a2(keep=runid group);
				by runid group;
			run;

			data table_&disttable.a;
				merge table_&disttable.a(in=a) 	
					  table_&disttable.a2(in=b);
				by runid group;

				if b and not a then do;
					total_count=0;
				end;
			run;

			/*Merge in totals*/
			data table_&disttable.a;	
				merge table_&disttable.a _total_bydp;
				by runid group;
			
				if missing(_mean)=0 then mean = strip(put(round(_mean, .01), 10.2));
				else mean='-';

				if missing(_std)=0 then std  = strip(put(round(_std, .01), 10.2));
				else std='-';

				min = strip(put(_min, comma12.0));
				p25 = strip(put(_p25, comma12.));
				median = strip(put(_median, comma12.));
				p75 = strip(put(_p75, comma12.));
				max = strip(put(_max, comma12.0));
				n = strip(put(total_count, comma12.0));

				array tmpvarn{*} min p25 median p75 max n;
					do i=1 to dim(tmpvarn);
					if tmpvarn{i} = '.' then tmpvarn{i}='-';
					end;

				drop i _: total_count;
			run;
		%end;
	%end;
	%else %do;
		%put Overall table not requested. Program will abort;
		%abort;
	%end;

	/*Loop through each stratification*/
	data _tablestrata;
		set &tablefile.(where=(table in (&table.) and tablesub ne 'A'));
		keep tablesub levelid:;
	run;
	proc sort data=_tablestrata nodupkey;
		by _all_;
	run;

	%isdata(dataset=_tablestrata);
	%let numstrat = &nobs.;

	%do strat = 1 %to &numstrat.;
		
		data _null_;
 			set _tablestrata;
			if _n_ = &strat. then do;
				call symputx('levelid1', levelid1);
				call symputx('levelid2', levelid2);
				call symputx('tablesub', tablesub);
			end;
		run;

		%if "&tablesub" = "B" %then %let stratvars = sex ;
		%if "&tablesub" = "C" %then %let stratvars = agegroup agegroupnum;
		%if "&tablesub" = "D" %then %let stratvars = race;
		%if "&tablesub" = "E" %then %let stratvars = hispanic;
		%if "&tablesub" = "BC" %then %let stratvars = sex agegroup agegroupnum;
		%if "&tablesub" = "BD" %then %let stratvars = sex race;
		%if "&tablesub" = "BE" %then %let stratvars = sex hispanic;
		%if "&tablesub" = "CD" %then %let stratvars = agegroup agegroupnum race;
		%if "&tablesub" = "CE" %then %let stratvars = agegroup agegroupnum hispanic;

		/*Extract column 1: Total*/
		proc sort data=&data. out=_total_bystrat(rename=&countvar.=total_count keep=runid group &stratvars. &countvar.);
			by runid group &stratvars.;
			where level in ("&levelid1.");
		run;

		/*Group continous var into categories*/
		%if "&cattable." ne "" %then %do;
			data _catdata;
				set &data.(where=(level in ("&levelid2.")));

				*assign strata;
				length strat $50.;
				%do c =1 %to &num_categories.;
					if %scan(&categories_boolean., &c., ' ') then do;
						strat = "%scan(&categories., &c., ' ')";
						stratsort = &c.;
					end;
				%end;

				%if %length(&categories)>0 %then %do;
				  if stratsort = . then do;
					strat = 'delete';
					stratsort = 999;
				  end;
				%end;
				%else %do;
				  stratsort=1;
				  strat='';
				%end;

				if strat='delete' then delete;
			run;

			*collapse across strata;
			proc means noprint nway data=_catdata;
				var &countvar.;
				class runid group &stratvars. strat stratsort / missing;
				output out=_distribution_cat_&tablesub.(drop=_:) sum=;
			run;

			*Transpose and square;	
			proc transpose data=_distribution_cat_&tablesub. out=_catdata_trans(drop=_NAME_) prefix=_;
				by runid group &stratvars.;
				var &countvar.;
				id stratsort;
			run;

			data table_&cattable.&tablesub._prepercent;
				merge _total_bystrat _catdata_trans;
				by runid group &stratvars.;
				
				*First ensure all category variables exist;
				if _n_ = 1 then do;
					dsid = open("_catdata_trans");
					%do c =1 %to &num_categories.;
						if varnum(dsid,"_&c.") = 0 then _&c. =.;
					%end;
					rc= close(dsid);
				end;
				drop rc dsid;

				*Fill in missing with 0, add labels;
				%do c =1 %to &num_categories.;
					if _&c. = . then _&c. = 0;
					label _&c. = "%scan(&categories., &c., ' ')";
				%end;
			run;
		%end;

		/*Continous var metrics*/
		%if "&disttable." ne "" %then %do;
			proc means data=&data.(where=(level in ("&levelid2."))) noprint nway;
				var &catvar.;
				freq &countvar.;
				class runid group &stratvars.;
				output out=table_&disttable.&tablesub.(drop=_type_ _freq_) p25=_p25 p75=_p75 
														max=_max 
														min=_min 
														median=_median 
														mean=_mean 
														std =_std;		
			run;	

			/*merge in squared table - freq statement drops rows where &countvar.=0*/
			proc sort data=&data.(where=(level = "&levelid2")) nodupkey out=table_&disttable.&tablesub.2(keep=runid group &stratvars.);
				by runid group &stratvars.;
			run;

			data table_&disttable.&tablesub.;
				merge table_&disttable.&tablesub.(in=a) 	
					  table_&disttable.&tablesub.2(in=b);
				by runid group &stratvars.;

				if b and not a then do;
					total_count=0;
				end;
			run;

			/*Merge in totals*/
			data table_&disttable.&tablesub._pretotal;	
				merge table_&disttable.&tablesub. _total_bystrat;
				by runid group &stratvars.;
			
				if missing(_mean)=0 then mean = strip(put(round(_mean, .01), 10.2));
				else mean='-';

				if missing(_std)=0 then std  = strip(put(round(_std, .01), 10.2));
				else std='-';

				min = strip(put(_min, comma12.0));
				p25 = strip(put(_p25, comma12.));
				median = strip(put(_median, comma12.));
				p75 = strip(put(_p75, comma12.));
				max = strip(put(_max, comma12.0));
				n = strip(put(total_count, comma12.0));

				array tmpvarn{*} min p25 median p75 max n;
					do i=1 to dim(tmpvarn);
					if tmpvarn{i} = '.' then tmpvarn{i}='-';
					end;

				drop i _: total_count;
			run;
		%end;
	%end;


	/*loop through each stratfication to merge in total rows*/
	%do strat = 1 %to &numstrat.;
		
		data _null_;
 			set _tablestrata;
			if _n_ = &strat. then do;
				call symputx('tablesub', tablesub);
			end;
		run;

		%if "&tablesub" = "B" %then %let stratvars = sex ;
		%if "&tablesub" = "C" %then %let stratvars = agegroup agegroupnum;
		%if "&tablesub" = "D" %then %let stratvars = race;
		%if "&tablesub" = "E" %then %let stratvars = hispanic;
		%if "&tablesub" = "BC" %then %let stratvars = sex agegroup agegroupnum;
		%if "&tablesub" = "BD" %then %let stratvars = sex race;
		%if "&tablesub" = "BE" %then %let stratvars = sex hispanic;
		%if "&tablesub" = "CD" %then %let stratvars = agegroup agegroupnum race;
		%if "&tablesub" = "CE" %then %let stratvars = agegroup agegroupnum hispanic;

		%if "&cattable" ne "" %then %do;

			*Merge in higher order stratifications;
			*all tables have overall - A table;
			data table_&cattable.&tablesub.;
				set table_&cattable.&tablesub._prepercent
				    table_&cattable.a(keep=runid group total_percent total_count
							%do c = 1 %to %to &num_categories.;
								_&c.
							%end;);
			run;

			*Compute percentiles;
			proc sql noprint undo_policy=none;
				create table table_&cattable.&tablesub. as
				select x.*
					  , y.total_count as overall_total
					 %do c = 1 %to %to &num_categories.;
					  , y._&c. as _total_&c.
					 %end;
				from table_&cattable.&tablesub. as x,
					 table_&cattable.a as y
				where x.group = y.group and x.runid = y.runid;
			quit;

			data table_&cattable.&tablesub.(drop=overall_total _total:);
				set table_&cattable.&tablesub. ;

				if overall_total >0 then do;
					total_percent = (total_count/overall_total)*100;
				end;
				else do;
					total_percent = 0;
				end;
				format total_percent 12.1;

				*Compute percent;
				%do c =1 %to &num_categories.;
					if _total_&c. >0 then do;
						_&c._percent = (_&c./_total_&c.)*100;
					end;
					else do;
						_&c._percent =0;
					end;
					format _&c._percent 12.1 _&c. comma12.0;
				%end;
			run;

			proc sort data=table_&cattable.&tablesub.;
				by runid group &stratvars.;
			run;

		%end;

		%if "&disttable" ne "" %then %do;
			data table_&disttable.&tablesub.;
				set table_&disttable.&tablesub._pretotal
				    table_&disttable.a;
			run;

			proc sort data=table_&disttable.&tablesub.;
				by runid group &stratvars.;
			run;
		%end;
			
		proc datasets nowarn noprint lib=work;
			delete _catdata_trans _catdata _total_bystrat _distribution_cat:;
		quit;

	%end;

	/*Apply display formatting*/

	%macro addlabelstodisttables(tablenum=);

		%do strat = 1 %to %eval(&numstrat.+1);

			%if %eval(&strat.=1) %then %do;
				%let tablesub = A;
			%end;
			%else %do;
				data _null_;
		 			set _tablestrata;
					if _n_ = %eval(&strat.-1) then do;
						call symputx('tablesub', tablesub);
					end;
				run;
			%end;

			%if "&tablesub" = "B" %then %let stratvars = sex ;
			%if "&tablesub" = "C" %then %let stratvars = agegroup;
			%if "&tablesub" = "D" %then %let stratvars = race;
			%if "&tablesub" = "E" %then %let stratvars = hispanic;
			%if "&tablesub" = "BC" %then %let stratvars = sex agegroup;
			%if "&tablesub" = "BD" %then %let stratvars = sex race;
			%if "&tablesub" = "BE" %then %let stratvars = sex hispanic;
			%if "&tablesub" = "CD" %then %let stratvars = agegroup race;
			%if "&tablesub" = "CE" %then %let stratvars = agegroup hispanic;

			/*All Tables - merge in exposure and order*/
			proc sql noprint undo_policy=none;
				create table table_&tablenum&tablesub. as
				select x.*,
	   				   y.exposure as groupn format=$&labellen.. length=&labellen.,
					   y.order,
					   0 as stratsort,
					   0 as stratsort2,
					   y.agegroupfmt
				from table_&tablenum&tablesub. as x
				left join &groupfile. as y
				on x.group = y.group and x.runid = y.runid;
			quit;

			/*One level strata*/
			/*Overall table takes exposure, stratification takes formatted label*/
			%if "&tablesub" ne "A" & %length(%sysfunc(strip(&tablesub.)))=1 %then %do;

				data table_&tablenum&tablesub.;
					 set table_&tablenum&tablesub.;

					 *recode groupn;
					 if missing(&stratvars.) = 0 then do;
					 	%if "&stratvars" = "sex" %then %do;
							if &stratvars='F' then do; groupn="Female"; stratsort=1; end;
              	   			if &stratvars='M' then do; groupn="Male";   stratsort=2; end;
				   			if &stratvars='O' then do; groupn="Other";  stratsort=3; end;
						%end;
					 	%if "&stratvars" = "agegroup" %then %do;
							if missing(agegroupfmt)=1 then do;
								groupn=strip(agegroup); 
								stratsort=agegroupnum; 
							end;
							else do;
								%do o = 1 %to &agefmtcnt.;
									%let o1 = %scan(&agefmtorder., &o.);
									if order = &o1. then do;
										groupn=strip(put(agegroup, $age&o1.fmt.));
										stratsort=agegroupnum; 
									end;
								%end;
							end;
						%end;
					 	%if "&stratvars" = "race" %then %do;
 							if &stratvars = 0 then do; groupn = "Unknown"; stratsort=1; end;
							if &stratvars = 1 then do; groupn = "American Indian/Alaska Native"; stratsort=2; end;
							if &stratvars = 2 then do; groupn = "Asian"; stratsort=3; end;
							if &stratvars = 3 then do; groupn = "Black/African American"; stratsort=4; end;
							if &stratvars = 4 then do; groupn = "Native Hawaiian/Other Pacific Islander"; stratsort=5; end;
							if &stratvars = 5 then do; groupn = "White"; stratsort=6; end;
 						%end;
					 	%if "&stratvars" = "hispanic" %then %do;
						    if &stratvars = "N" then do; groupn = "Not Hispanic Origin"; stratsort=1; end;
						    if &stratvars = "Y" then do; groupn = "Hispanic Origin"; stratsort=2; end;
							if &stratvars = "U" then do; groupn = "Unknown"; stratsort=3; end;
						%end;
					end;

					drop agegroupfmt;
				run;

			%end;

			/*Two level strata*/
			%if "&tablesub" ne "A" & %length(%sysfunc(strip(&tablesub.)))=2 %then %do;

				%let firststrat = %scan(&stratvars., 1);
				%let secondstrat = %scan(&stratvars., 2);

				%let agegroupnum = ;
				%if "&firststrat" = "agegroup" %then %do;
					%let agegroupnum = , agegroupnum , agegroupfmt;
				%end;

				/*Create header row*/
				proc sql noprint;
					create table table&tablenum.&tablesub._headerrow as
					select distinct runid, 
									group,
									order, 
									stratsort, 
									stratsort2,
									&firststrat.
									&agegroupnum.
					from table_&tablenum&tablesub.
					where missing(&firststrat.)=0;
				quit;

				data table_&tablenum&tablesub.;
					 set table_&tablenum&tablesub. table&tablenum.&tablesub._headerrow;

					 *First stratification always sex or age group;
					 *recode groupn;
						 %if "&firststrat" = "sex" %then %do;
						 	if &firststrat='F' then do; groupn="Female"; stratsort=1; end;
              	   			if &firststrat='M' then do; groupn="Male";   stratsort=2; end;
				   			if &firststrat='O' then do; groupn="Other";  stratsort=3; end;
						%end;
						%if "&firststrat" = "agegroup" %then %do;
							if missing(agegroup) = 0 then do;
								if missing(agegroupfmt)=1 then do;
									groupn=strip(agegroup); 
									stratsort=agegroupnum; 
								end;
								else do;
									%do o = 1 %to &agefmtcnt.;
										%let o1 = %scan(&agefmtorder., &o.);
										if order = &o1. then do;
											groupn=strip(put(agegroup, $age&o1.fmt.));
											stratsort=agegroupnum; 
										end;
									%end;
								end;
							end;
						%end;

					if missing(&firststrat.) = 0 and missing(&secondstrat.) =0 then do;
					 	%if "&secondstrat" = "agegroup" %then %do;
							if missing(agegroup) = 0 then do;
								if missing(agegroupfmt)=1 then do;
									groupn=strip(agegroup); 
									stratsort2=agegroupnum; 
								end;
								else do;
									%do o = 1 %to &agefmtcnt.;
										%let o1 = %scan(&agefmtorder., &o.);
										if order = &o1. then do;
											groupn=strip(put(agegroup, $age&o1.fmt.));
											stratsort2=agegroupnum; 
										end;
									%end;
								end;
							end;
						%end;
					 	%if "&secondstrat" = "race" %then %do;
 							if &secondstrat = 0 then do; groupn = "Unknown"; stratsort2=1; end;
							if &secondstrat = 1 then do; groupn = "American Indian/Alaska Native"; stratsort2=2; end;
							if &secondstrat = 2 then do; groupn = "Asian"; stratsort2=3; end;
							if &secondstrat = 3 then do; groupn = "Black/African American"; stratsort2=4; end;
							if &secondstrat = 4 then do; groupn = "Native Hawaiian/Other Pacific Islander"; stratsort2=5; end;
							if &secondstrat = 5 then do; groupn = "White"; stratsort2=6; end;
 						%end;
					 	%if "&secondstrat" = "hispanic" %then %do;
						    if &secondstrat = "N" then do; groupn = "Not Hispanic Origin"; stratsort2=1; end;
						    if &secondstrat = "Y" then do; groupn = "Hispanic Origin"; stratsort2=2; end;
							if &secondstrat = "U" then do; groupn = "Unknown"; stratsort2=3; end;
						%end;
					end;
					drop agegroupfmt;
				run;

			%end;

			proc sort data=table_&tablenum&tablesub.;
				by order stratsort stratsort2;
			run;

			proc datasets nowarn noprint lib=work;
				delete table&tablenum.&tablesub._headerrow;
			quit;
		%end;

	%mend;

	%if "&cattable" ne "" %then %do;
		%addlabelstodisttables(tablenum=&cattable.);
	%end;
	%if "&disttable" ne "" %then %do;
		%addlabelstodisttables(tablenum=&disttable.);
	%end;

	/*clear temp datasets*/
	proc datasets nowarn noprint lib=work;
		delete _total: _table_a _tablecheck: _tablestrata;
	quit;


	%put =====> END MACRO: ms_t5_create_distribution_tables v2.1;

%mend ms_t5_create_distribution_tables;

