****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_t5_set_reportparameters.sas  
*
* Created (mm/dd/yyyy): 07/24/2018
* Last modified: 02/06/2019 
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This macro reads in inputfile data to set macro parameters for %ms_t5_create_t5_report
*   
*  Program inputs:                                                                                   
* 
*  Program outputs:                                                                                
* 
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1		  2/6/2019	 AP			Added Figure 5 parameter check, removed lasttablenum
*
***************************************************************************************************;

%macro ms_t5_set_reportparameters;

	%put =====> MACRO CALLED: ms_t5_set_reportparameters v1.1;

	%global runidlist num_allgroups tablelist allgroups labellen lasttablenum maxorder figure5label figure5count;

	%let runidlist=;
	%let grouplist=;
	%let orderlist=;
	%let labellen =50;
	%let figure5label = ;
	%let figure5count = 0;

	/*************************/
	/* Table macro variables */
   	/*************************/

	*List of tables to produce;
	proc sql noprint;
		select distinct table into: tablelist separated by ' '
		from &tablefile.;
	quit;

	/***********/
	/* Figures */
	/***********/

	/*check for censoring display*/
	%if %sysfunc(prxmatch(m/F5/i,&tablelist.)) > 0 %then %do;
		%if %length(&censoring_display_F5) = 0 | &censoring_display_F5 eq %str() | %sysfunc(countw(&censoring_display_F5.)) ne 1 %then %do;
			%put WARNING: censoring_display_F5 does not exist or more than ONE censoring is specified. Figure5 will not be created;
			%let tablelist = %sysfunc(compbl(%sysfunc(tranwrd(&tablelist., F5,))));
		%end;
	%end;

	*Figure5Count;
	proc sql noprint;
		select count(*) into: figure5count
		from &groupfile.
		where output_figure5 = 'Y';
	quit;

	/*Figure attributes*/
	%global colors clrCnt;
	%let colors = darkblue darkgreen darkpurple darkred darkorange black brown magenta darkyellow lightblue lightgreen
			lightpurple lightred lightorange lightyellow lightgray darkgray lightbrown;
	%let clrCnt = %sysfunc(Countw(&colors));


	/*************************/
	/* Group macro variables */
   	/*************************/

	*Create a list of groups within runIDs;
	proc sql noprint;
	    select distinct runid into: runidlist separated by " "
	    from &groupfile.
	quit;
	%put &runidlist.;

	%do r = 1 %to %sysfunc(countw(&runidlist.));
		%global runid&r. grouplist&r.;
		%let runid&r. = %scan(&runidlist., &r.);

		proc sql noprint;
			select "'"||STRIP(group)||"'" into: grouplist&r. separated by ','
			from &groupfile.
			where runid = "&&runid&r.";
		quit;
		%put grouplist for &&runid&r. = &&grouplist&r.;
	%end;

	proc sql noprint;  
		*total number of groups;
		select count(*) into: num_allgroups
	    from &groupfile.;

		select max(order) into: maxorder
		from &groupfile.;

		/* maxlength of exposure */
		select max(length(exposure))+50 into: labellen
		from &groupfile.;
	quit;
	%let labellen = &labellen.;

	/*Put exposure label into macro vars*/
	%do a=1 %to &num_allgroups.;
		%global exposure&a.;
		data _null_;
			set &groupfile.(where=(order=&a.));
			if _n_ = 1 then do;
				call symputx("exposure&a.", exposure);
			end;
		run;
	%end;

	/* Age Groupings - user-defined age group formats*/
	proc sort data=&groupfile.(where=(missing(agegroupfmt)=0)) out=agefmt;
	    by order;
    run;

	%global agefmtorder agefmtcnt agefmtgrp;
	%let agefmtorder=;
	%let agefmtgrp=;
	%let agefmtcnt=0;

	%isdata(dataset=agefmt);
    %if &nobs>0 %then %do;
		proc sql noprint;
        	select distinct order, group into: agefmtorder separated by " ", :agefmtgrp separated by " "
        	from agefmt
        	order by order;
       	quit;
	   	%put &agefmtorder. &agefmtgrp;

	   	%let agefmtcnt=%sysfunc(countw(&agefmtorder));
      	%do fmtcnt=1 %to &agefmtcnt.;
        	%global tmpfmt&fmtcnt agefmt&fmtcnt;
			%let pick=%scan(&agefmtorder.,&fmtcnt.);
		    data _null_;
			    set agefmt;
			    if order=&pick.;
		        tmpfmt='age'||strip(order)||'fmt';
		        call symputx("tmpfmt&fmtcnt", tmpfmt);
			    call symputx("agefmt&fmtcnt", agegroupfmt);
			run; 
			%put &&tmpfmt&fmtcnt agefmt&fmtcnt;
		%end;

		proc format;
		%do fmtcnt=1 %to &agefmtcnt.;
			value $&&tmpfmt&fmtcnt. &&agefmt&fmtcnt.
			other = "Other Age";
	    %end;
		run;
	%end;

	/**************************/
	/* Report macro variables */
   	/**************************/

	/*Assign default cellpadding value*/
	%if %length(&line_spacing.) = 0 %then %do;
		%let line_spacing = 1.75;
	%end;

	/*print logo*/
	%global report_header;
	%let report_header = title1 j=L font=arial h=12pt "Confidential ^n Do Not Distribute" ;	
	%if %sysevalf(%superq(logo)=,boolean)=0 %then %let report_header = title j=R '^S={preimage="&infolder./&logo."}' j=L font=arial h=11pt "^S={vjust=m}Confidential ^n Do Not Distribute" ;


     %put =====> END MACRO: ms_t5_set_reportparameters v1.1;

%mend ms_t5_set_reportparameters;
